# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s):
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: $
"""Interface to the Reactome webs services

:STATUS: in progress/draft don't use !

.. topic:: What is Reactome?

    :URL: http://www.reactome.org/ReactomeGWT/entrypoint.html
    :Citation: http://www.reactome.org/citation.html
    :WSDL: http://www.reactome.org:8080/caBIOWebApp/services/caBIOService?wsdl

    .. highlights::

        "REACTOME is an open-source, open access, manually curated and peer-reviewed
        pathway database. Pathway annotations are authored by expert biologists, in
        collaboration with Reactome editorial staff and cross-referenced to many
        bioinformatics databases. These include NCBI Entrez Gene, Ensembl and UniProt
        databases, the UCSC and HapMap Genome Browsers, the KEGG Compound and ChEBI
        small molecule databases, PubMed, and Gene Ontology. ... "

        -- from Reactome web site

"""


from services import WSDLService, RESTService
import webbrowser
import copy



class ReactomeURL(RESTService):

    _url = "http://www.reactome.org/cgi-bin"
    def __init__(self, verbose=True):
        super(ReactomeURL, self).__init__("Reactome(URL)",url=ReactomeURL._url,
            verbose=verbose)

    def link(self, source, identifier):
        """Linking to Reactome

        This can be achieved by creating URLs containing the name of and
        an identifier from an "external" database in the following format::

            r.link("COMPOUND", "C00002") #  COMPOUND identifiers, e.g. COMPOUND:C00002
            r.link("UNIPROT", "P30304")  #  UniProt accession numbers and identifiers, e.g. UNIPROT:P30304
            r.link("CHEBI", "15422")     #  ChEBI identifiers, e.g. CHEBI:15422

        """
        url = self.url + "/link?SOURCE" +  source + "&ID=" + identifier
        res = self.request(url)
        return res



class Reactome(WSDLService):
    """Interface to the `Reactome <http://www.reactome.org>`_ service

    :in progress: dont use


    """
    _url = "http://www.reactome.org:8080/caBIOWebApp/services/caBIOService?wsdl"
    def __init__(self, verbose=True):
        """Constructor

        :param bool verbose:

        """
        super(Reactome, self).__init__(name="Reactome", url=Reactome._url, verbose=verbose)

    def version(self):
        r = WSDLService("version", "http://www.reactome.org:8080/caBIOWebApp/services/Version?wsdl")
        res = r.serv.getVersion()
        print res

    def queryPathwaysForReferenceIdentifiers(self, list_ids):
        """

            res = r.queryPathwaysForReferenceIdentifiers(["Q9Y266", "P17480", "P20248"])

        """
        results = self.serv.queryPathwaysForReferenceIdentifiers(list_ids)

        # We could return the raw results but it is not really readable.
        # Let use ReactomePathway class
        queries = []
        for res in results:
            queries.append(ReactomePathway(res))
        return queries

    def generatePathwayDiagramInSVG(self, Id):
        return self.serv.generatePathwayDiagramInSVG(Id)


    def listTopLevelPathways(self):
        """

            res = r.listTopLevelPathways()
            [x.id for x in res]

        """
        return self.serv.listTopLevelPathways()

    def getMaxSizeInListObjects(self):
        res = self.serv.getMaxSizeInListObjects
        return res

"""
[u'generatePathwayDiagramInSVG',
 u'generatePathwayDiagramInSVGForId',
 u'listByQuery',
 u'listObjects',
 u'listPathwayParticipants',
 u'listPathwayParticipantsForId',
 u'listTopLevelPathways',
 u'loadPathwayForId',
 u'loadPathwayForObject',

 u'queryById',
 u'queryByIds',
 u'queryByObject',
 u'queryByObjects',
 u'queryPathwaysForEntities',
 u'queryPathwaysForEntityIds',
"""





#caBIOService: http://www.reactome.org:8080/caBIOWebApp/services/caBIOService?wsdl

class ReactomeBioPAXExporter(WSDLService):
    """http://www.reactome.org/entitylevelview/PathwayBrowser.html#DB=gk_current&FOCUS_SPECIES_ID=48887&FOCUS_PATHWAY_ID=177929&ID=177929"""

    def __init__(self, verbose=True, debug=False, url=None):
        self.url = "http://www.reactome.org:8080/caBIOWebApp/services/BioPAXExporter?wsdl"
        super(ReactomeBioPAXExporter, self).__init__(name="ReactomeBioPAXExporter",
            url=self.url, verbose=verbose)




class ReactomePathway(object):

    def __init__(self, entry):
        self.raw_data = copy.deepcopy(entry)
        # just adding the attributes to make life easier
        for k in self.raw_data._keyord:
            setattr(self, k, getattr(self.raw_data, k))


    def __str__(self):

        txt = "id: " + str(self.id)
        txt += "\nname: " + str(self.name)

        txt += "\nhasComponent:"
        if self.hasComponent:
            txt += str(len(self.hasComponent))



        if self.raw_data.compartment:
            txt += "\ncompartment:" + str(len(self.raw_data.compartment))
        else:
            txt += "\ncompartment: " + str(self.raw_data.compartment)

        txt += "\nliteratureReference:"
        this = self.raw_data.literatureReference
        if this:
            for i,x in enumerate(this):
                txt += " --- %s: %s %s\n" % (i, x.id, x.name)

        txt += "\nspecies:"
        this = self.raw_data.species
        if this:
            txt += "  %s (%s)" % (this.scientificName, this.id)

        txt += "\nsummation:"
        this = self.raw_data.summation
        if this:
            for x in this:
                txt += " - %s \n" % (self.raw_data.id)

        txt += "\ngoBiologicalProcess:"
        this = self.raw_data.goBiologicalProcess
        if this: txt += "\n - %s %s\n" % (this.id, this.name)

        txt += "\ninferredFrom:" + str(self.raw_data.inferredFrom)
        txt += "\northologousEvent: "
        this = self.raw_data.orthologousEvent
        if this:
            txt += str(len(this))

        txt += "\nprecedingEvent: " + str(self.raw_data.precedingEvent)
        txt += "\nevidence Type: " + str(self.raw_data.evidenceType) + "\n"

        return txt




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: kegg.py 55 2013-01-11 17:17:31Z cokelaer $
"""This module provides a class :class:`~Kegg` that allows an easy access to all the
WDSL Kegg interface as well as additional methods to obtain statistics about the
Kegg database. See below for details about the functionalities available and the
tutorial/quickstart :ref:`kegg_tutorial`.

.. seealso:: `<http://www.kegg.jp/kegg/soap/doc/keggapi_manual.html>`_

Some terminology 
--------------------

This list is taken from Kegg API page and slightly edited.


* organisms (**org**) are made of a three-letter (or four-letter) code (e.g., hsa
  stands for Human Sapiens) used in KEGG (see
  :meth:`~bioservices.kegg.Kegg.organisms`). 
* **db** is a database name used in GenomeNet service. See
  :meth:`~bioservices.kegg.Kegg.databases`
* **entry_id** is a unique identifier that is a combination of the database name
  and the identifier of an entry joined by a colon sign (e.g. 'embl:J00231'
  means an EMBL entry 'J00231').
  **entry_id** includes:

    * **genes_id**: identifier consisting of 'keggorg' and a gene name (e.g. 'eco:b0001' means an E. coli gene 'b0001').
    * **enzyme_id**: identifier consisting of database name 'ec' and an enzyme code used in KEGG/LIGAND ENZYME database. (e.g. 'ec:1.1.1.1' means an alcohol dehydrogenase enzyme)
    * **compound_id**: identifier consisting of database name 'cpd' and a
      compound number used in KEGG COMPOUND / LIGAND database (e.g. 'cpd:C00158'
      means a citric acid). Some compounds also have 'glycan_id' and
      both IDs are accepted and converted internally.
    * **drug_id**: identifier consisting of database name 'dr' and a compound
      number used in KEGG DRUG / LIGAND database (e.g. 'dr:D00201' means a
      tetracycline).
    * **glycan_id**: identifier consisting of database name 'gl' and a glycan
      number used in KEGG GLYCAN database (e.g. 'gl:G00050' means a
      Paragloboside). Some glycans also have 'compound_id' and both
      IDs are accepted and converted internally.
    * **reaction_id**:  identifier consisting of database name 'rn' and a
      reaction number used in KEGG/REACTION (e.g. 'rn:R00959' is a reaction
      which catalyze cpd:C00103 into cpd:C00668).
    * **pathway_id**: identifier consisting of 'path' and a pathway number used
      in KEGG/PATHWAY. Pathway numbers prefixed by 'map' specify the reference
      pathway and pathways prefixed by the 'keggorg' specify pathways specific
      to the organism (e.g. 'path:map00020' means a reference pathway for the
      cytrate cycle and 'path:eco00020' means a same pathway of which E. coli
      genes are marked).
    * **motif_id** is a motif identifier consisting of motif database names
      ('ps' for prosite, 'bl' for blocks, 'pr' for prints, 'pd' for prodom, and
      'pf' for pfam) and a motif entry name. (e.g. 'pf:DnaJ' means a Pfam
      database entry 'DnaJ').
    * **ko_id**: identifier consisting of 'ko' and a ko number used in KEGG/KO.
      KO (KEGG Orthology) is an classification of orthologous genes defined by
      KEGG (e.g. 'ko:K02598' means a KO group for nitrite transporter NirC
      genes).
    * **ko_class_id**: identifier which is used to classify 'ko_id' hierarchically 
      (e.g. '01110' means a 'Carbohydrate Metabolism' class).
      `URL:http://www.genome.jp/dbget-bin/get_htext?KO`   
* **offset** and 'limit' are both an integer and used to control the number of
  the results returned at once. Methods having these arguments will return
  first 'limit' results starting from 'offset'th.
* **fg_color_list** is a list of colors for the foreground (corresponding to the
  texts and borders of the objects on the KEGG pathway map).
* **bg_color_list** is a list of colors for the background (corresponding to the
  inside of the objects on the KEGG pathway map).


Functions/methods
-------------------

In addition to all Kegg API methods (see :meth:`~bioservices.kegg.Kegg.methods`),
we provide additional functionalities:

.. autosummary::

    bioservices.kegg.Kegg.analyse
    bioservices.kegg.Kegg.lookfor_specy
    bioservices.kegg.Kegg.lookfor_pathway
    bioservices.kegg.Kegg.lookfor_organism
    bioservices.kegg.Kegg.extra_pathway_summary
    bioservices.kegg.Kegg.extra_get_all_relations
    bioservices.kegg.Kegg.extra_get_pathway_types
    bioservices.kegg.Kegg.extra_get_elements_by_pathway

"""


from services import WSDLService
import webbrowser
import copy


class Items(object):
    """Simple class to ease access to some data in the Kegg database


    Used to access to organisms and databases. Could be extended for the
    pathways

    ::

            k = Kegg()
            k.organisms
            k.organisms.items
            k.organisms.entry_id
            k.organisms.definition

    """

    def __init__(self, data, name="items", verbose=True):
        self.verbose = verbose
        self.name = name

        self._items = copy.deepcopy(data)

    def _get_definitions(self):
        ids = [x['definition'] for x in self.items]
        return ids
    definitions = property(_get_definitions, doc="return list of definition")

    def _get_entry_ids(self):
        ids = [x['entry_id'] for x in self.items]
        return ids
    entry_ids = property(_get_entry_ids, doc="return list of Ids")

    def _get_items(self):
        return self._items
    items = property(_get_items, doc=_get_items.__doc__)

    def lookfor(self, query):
        """Search for a pattern in the items

        :param str query: can be an id, a word

        case insensitive
        """
        matches = []
        for i, item in enumerate(self.definitions):
            if query.lower() in item.lower():
                matches.append(i)
        for i, item in enumerate(self.entry_ids):
            if query.lower() in item.lower():
                matches.append(i)
        return [(self.definitions[i], self.entry_ids[i]) for i in matches]

    def __str__(self):
        txt = ""
        txt += "%60s || %s\n" % ("Definition", "ID")
        txt += "="*80 + "\n"
        for x,y in zip(self.definitions, self.entry_ids):
            txt +=  "%60s || %s\n" % (x,y)
        return txt 



class Kegg(WSDLService):
    """Interface to the `KEGG <http://www.genome.jp/kegg/pathway.html>`_ database

    ::

        import kegg
        k = kegg.Kegg()
        k.methods
        k.bget("hsa:7535")

        print len(k.pathways)

        k.organism #hsa (homo sapiens by default)

        .. warning:: unfortunately, the WSDL sevices is closed since 31st Dec 2012

    """
    _url = "http://soap.genome.jp/KEGG.wsdl"
    def __init__(self, verbose=True):
        """Constructor

        :param bool verbose:

        """
        import logging
        logging.warning("unfortunately, the WSDL sevices is closed since 31st Dec 2012")
        """super(Kegg, self).__init__(name="Kegg", url=Kegg._url, verbose=verbose)
        self._pathways = {}
        self._buffer = {}
        self._relations = {}

        self._mapping = None
        self._organism = 'hsa'
        self.debug = debug
        if self.debug: 
            self.info()

        if self.verbose:
            print "Fetching databases and organisms..."
        self.databases = Items(self.serv.list_databases(), "databases", self.verbose)
        self.organisms = Items(self.serv.list_organisms(), "organisms", self.verbose)
        """

    def _check_pathway_id(self, pathway_id):
        pids = [x.entry_id for x in self.pathways]
        if pathway_id not in pids:
            if len(self.bget(pathway_id))==0:
                print "%s pathway if cannot be found" % pathway_id
                raise Exception



    def info(self):
        """For debugging. 

        prints the WSDL service methods not yet wrapped."""
        methods = self.methods[:]
        for m in methods:
            if m not in Kegg.__dict__.keys():
                print "%s not part of the methods but can be access through server attribute" % m
                # self.__setattr__(m, self.serv.__getattr__(m)) # this code
                # makes it available, but we must first call the
                # self.server.method to allow self.method to work...

    def _set_organism(self, organism):
        if organism in self.organisms.entry_ids:
            self._organism = organism
        else:
            raise Exception("%s organism not valid. See organisms attribute" % organism)
    def _get_organism(self):
        return self._organism
    organism = property(_get_organism, _set_organism, 
        doc="read/write attribute for the organism")


    def get_pathways_by_genes(self, gene_id):
        """Returns pathways that contains the gene_id

        :param str gene_id: a valid **gene_id**

        call WSDL
        """
        if isinstance(gene_id, str):
            res = self.serv.get_pathways_by_genes([gene_id])
        else:
            res = self.serv.get_pathways_by_genes(list(gene_id))
        return res

    def get_pathways_by_reactions(self, reactions_list):
        """

        """
        if isinstance(reactions_list, str):
            res = self.serv.get_pathways_by_reactions([reactions])
        else:
            res = self.serv.get_pathways_by_reactions(list(reactions))
        return res

    def get_pathways_by_compounds(self, compounds_list):
        """Search all pathways which include all the given compounds.

        ::

            >>> k.serv.get_pathways_by_compounds(['cpd:C00076', 'cpd:C00165', 'cpd:C01245'])
        """
        return k.serv.get_pathways_by_compounds(compounds_list)

    def get_pathways_by_enzymes(self, enzymes_list):
        """Search all pathways which include all the given enzymes.

        :param list enzymes_list:

        """
        if isinstance(enzymes_list, str):
            res = self.serv.get_pathways_by_enzynes([enzymes_list])
        else:
            res = self.serv.get_pathways_by_enzymes(list(enzymes_list))
        return res

    def get_pathways_by_drugs(self, drugs_list):
        """Search all pathways which include all the given drugs.

        :param list enzymes_list:

        """
        if isinstance(drugs_list, str):
            res = self.serv.get_pathways_by_drugs([drugs_list])
        else:
            res = self.serv.get_pathways_by_drugs(list(drugs_list))
        return res

    def get_pathways_by_glycans(self, glycans_list):
        """Search all pathways which include all the given glycans.

        """
        if isinstance(glycans_list, str):
            res = self.serv.get_pathways_by_glycans([glycans_list])
        else:
            res = self.serv.get_pathways_by_glycans(list(glycans_list))
        return res

    def get_pathways_by_kos(self, kos_list):
        """Retrieve all pathways of the organisms which include all the given KO IDs.

        """
        if isinstance(kos_list, str):
            res = self.serv.get_pathways_by_kos([kos_list])
        else:
            res = self.serv.get_pathways_by_kos(list(kos_list))
        return res


    def get_compounds_by_pathway(self, pathway_id):
        """Search all compounds on the specified pathway.

        :param str pathway_id: a valid pathway id

        ::

            >>>        k.get_compounds_by_pathway(pathway_id)
            ['cpd:C00076', 'cpd:C00165', 'cpd:C01245']
        """
        self._check_pathway_id(pathway_id)
        res = self.serv.get_compounds_by_pathway(pathway_id)
        return res


    def get_drugs_by_pathway(self, pathway_id):
        """Search all drugs on the specified pathway.

        :param str pathway_id: a valid pathway id

        """
        self._check_pathway_id(pathway_id)
        res = self.serv.get_drugs_by_pathway(pathway_id)
        return res

    def get_glycans_by_pathway(self, pathway_id):
        """Search all glycans on the specified pathway.

        :param str pathway_id: a valid pathway id
        """
        self._check_pathway_id(pathway_id)
        return self.serv.get_glycans_by_pathway(pathway_id)

    def get_enzymes_by_pathway(self, pathway_id):
        """Retrieve all enzymes on the specified pathway.

        :param str pathway_id: a valid pathway id
        """
        self._check_pathway_id(pathway_id)
        return self.serv.get_enzymes_by_pathway(pathway_id)

    def get_linked_by_pathway(self, linked):
        """

        :param str pathway_id: a valid pathway id
        """
        self._check_pathway_id(pathway_id)
        return self.serv.get_linked_by_pathway(pathway_id)



    def binfo(self, db=None, verbose=True):
        """show information of the latest GenBank database

        :param str db: name of a specific database. default is None (fetch all DBs)
        :param bool verbose: print the results

        ::

            results = k.binfo("gb", verbose=True)
        """
        if db==None:
            res = self.serv.binfo()
            print res
        else:
            res = self.serv.binfo(db)
            print res
        return res

    def bget(self, entry_id):
        """returns complete information about an entry_id

        ::
            
            # Retrieve the human gene with 7535
            res = k.bget("hsa:7535")
            # retrieve two KEGG/GENES entries
            bget("eco:b0002 hin:tRNA-Cys-1")
            # retrieve nucleic acid sequences in a FASTA format
            bget("-f -n n eco:b0002 hin:tRNA-Cys-1")
            # retrieve amino acid sequence in a FASTA format
            bget("-f -n a eco:b0002")

        """
        assert type(entry_id)==str or type(entry_id)==list
        if type(entry_id) != list:
            return self.serv.bget(entry_id)
        else:
            results = []
            for e in entry_id:
                res = self.serv.bget(e)
                results.append(res)
            return results

    def bfind(self, request):
        """Searches entries by keywords. 

        User need to specify a database from those which are supported by
        DBGET system before keywords. Number of keywords given at a time is restricted
        up to 100.
    
        :param str request: a string that specifies a database. Number of 
            keywords must be less than 100
        :return: string

        ::

            # Returns the IDs and definitions of entries which have definition
            # including the word 'E-cadherin' and 'human' from GenBank.
            k = Kegg()
            k.bfind("gb E-cadherin human")

        """
        return self.serv.bfind(request)


    def btit(self, request):
        """btit is used for retrieving the definitions by given database entries. 

        Number of entries given at a time is restricted up to 100.

        :param str request: a string contains valid DB entries
        :return: string

        ::

            # Returns the ids and definitions of four GENES entries "hsa:1798",
            # "mmu:13478", "dme:CG5287-PA" and cel:Y60A3A.14".
            k.btit("hsa:1798")

        """
        return self.serv.btit(request)



    def get_number_of_genes_by_organism(self, organism=None):
        if organism == None:
            organism = self.organism
        if organism not in self.organisms.entry_ids:
            raise Exception("invalid organism id. See organism_ids attribute.")

        if organism in self._buffer.keys():
            return self._buffer[organism]['N']
        else:
            N = self.serv.get_number_of_genes_by_organism(organism)
            self._buffer[organism] = {'N': N}
            return N

    def list_pathways(self, organism=None):

        if organism == None:
            return self.serv.list_pathways(self.organism)
        else:
            return self.serv.list_pathways(organism)

    def _get_pathways(self):
        """Returns names of the organisms supported by KEGG databases

        ::

            k = Kegg()
            k.pathways()
            k.list_pathways()
        """
        if self.organism not in self._pathways.keys():
            if self.verbose:
                print "fetching pathways...",
            res = self.serv.list_pathways(self.organism)
            self._pathways[self.organism] = res
            if self.verbose:
                print "ok"
        return self._pathways[self.organism]
    pathways = property(_get_pathways)




    def bconv(self, entry_ids):
        """Given a gene identifier, the functions queries KEGG to retrieve the appropriate KEGG ID.

        The bconv command converts external IDs to KEGG IDs. Currently, following
        external databases are available.

        :param list entry_ids: ids can be a list of strings or a single string where ids are separated by spaces

        ::

            >>> print k.bconv("ncbi-gi:10047086 uniprot:P43403")
            ncbi-gi:10047086    hsa:54206   equivalent
            up:P43403   hsa:7535    equivalent
            >>> print k.bconv(["ncbi-gi:10047086", "uniprot:P43403"])
            ncbi-gi:10047086    hsa:54206   equivalent
            up:P43403   hsa:7535    equivalent


        Note that the original Kegg API works with a unique string. However, we
        also implemented a way to provide a list of strings.


        """
        if isinstance(entry_ids, list)==True:
            request = " ".join([x for x in entry_ids])
        else:
            request = entry_ids[:]
        return self.serv.bconv(request)



    def retrieveKGML(self, pathway_id="05210", organism="hsa", filename="test.xml", 
            format=None):
        """Return URL of a pathway


        :param str pathway_id: just the id, not the organism !
        :param str organism: 
        :param str filename: 
        :param str format: e.g., kgml 
            

        http://www.genome.jp/dbget-bin/show_pathway?hsa05210


        """
        self._check_pathway_id(":".join(["path", organism+ pathway_id]))

        if format == None:
            url = "http://www.genome.jp/dbget-bin/show_pathway?%s%s" 
            url = url % (organism, pathway_id)
        elif format == "KGML" or format == "kgml":
            url = "http://www.genome.jp/kegg-bin/download?entry=%s%s&format=kgml"
            url = url % (organism, pathway_id)
        else:
            raise NotImplementedError("only kgml format implemented")
        return url


    def analyse(self, entry_id, type="gene"):
        """Analyse the contents returned by :meth:`bget`

        :param str entry_id: a valid **entry_id** identifier
        :param str type: default is "gene"
        :return: a dictionary with NAME/DESCRIPTION/PATHWAY keys found in the
            output of bget method.

        Example::

            >>> k = Kegg()
            >>> k.analyse("hsa:7535")
            {'DEFINITION': 'zeta-chain (TCR) associated protein kinase 70kDa (EC:2.7.10.2)',
             'NAME': ['ZAP70', 'SRK', 'STD', 'TZK', 'ZAP-70'],
             'PATHWAY': ['hsa04064  NF-kappa B signaling pathway',
              'hsa04650  Natural killer cell mediated cytotoxicity',
              'hsa04660  T cell receptor signaling pathway',
              'hsa05340  Primary immunodeficiency']}


        .. note:: only the type "gene" is implemented so far. The gene case
            analyse only a subset of the returned data string to extract the
            names or description and pathways.

        """
        if type=="gene": 
            data = self.bget_buffer(entry_id)
            return self._analyse_gene(data)
        else:
            raise NotImplementedError

    def bget_buffer(self, entry_id):
        """Same as bget but buffers the results.


        """
        if entry_id not in self._buffer:
            data = self.bget(entry_id)
            self._buffer[entry_id] = data
        return self._buffer[entry_id]

    def _analyse_gene(self, data):
        output = {}
        try:
            names = [x for x in data.split("\n") if x.startswith("NAME")][0]
            names = names.split("NAME")[1].strip()
            names = [x.strip() for x in names.split(",")]
            output['NAME'] = names
        except:
            pass

        try:
            # we can also use get_pathways_by_gene()
            output['PATHWAY'] = [x.strip() for x in data.split("PATHWAY")[1].split("\n") if x.strip().startswith("hsa")]
        except:
            pass

        try:
            defi = [x.strip() for x in data.split("\n") if x.startswith("DEFINITION")]
            defi = defi[0].split("DEFINITION")[1].strip()

            output['DEFINITION'] = defi
        except:
            pass

        return output


    def get_genes_by_motifs(self, genes, start, max_results):
        """Onbtain the names of genes that contain the motifs


        Given a set of motif ids, the function searches the databases implied by the motif ids for genes
        containing the motifs specified by the motif ids

        :param genes: list of character strings for the ids of the motifs that are conserved by genes across organisms
        :param int start: location of the entry in the query results from which the results will be extracted and returned
        :param int max_results: maximum number of entries that will be extracted from the query results and returned


        KEGG seems to have two ways of defining the ids for motifs. One is the motif ids obtained through
        get.motifs.by.gene, where pfam, tfam, pspt, pspf are used for the Pfam, TIGRFAM, PROSITE
        pattern, and PROSITE profile database, respectively and for the first part of a motif id (e. g.
        pfam:aakinase). Another is the motif ids used to query the databases for genes that contain the
        motif, where only the first two letters of the abbreviations for databases form the first part of a motif
        id (e. g. pf:aakinase)



        :return: The function returns a named vector with the names of the vector being the
            textual definition of genes and values of the vector being the ids used by KEGG 
            to represent genes.

        ::

            k = Kegg():
            k.serv.get_genes_by_motifs(["pf:DnaJ", "ps:DNAJ_2"], 1, 10)


        :source: KEGGSOAP for the doc

        """

        return self.serv.get_genes_by_motifs(genes, start, max_results)

    def get_genes_by_organism(self, organism="hsa", start=1, max_results=1e9):
        """Obtain list of genes for a given organism


        :param str organism: id used by KEGG for organisms. The organism
            ids are normally three-letter codes with the first letter being the first letter
            of the genus name and the rest being the first two letters of the species name of the
            scientic name of the organism of concern
        :param int start: location of the entry in the query results from which the results will be extracted and returned
        :param int max_results: maximum number of entries that will be extracted from the query results and returned


        :return: list of ids used by KEGG ro represent genes

        """
        if organism in self._buffer.keys():
            if "genes" in self._buffer[organism].keys():
                return self._buffer[organism]['genes']
            else:
                res = self.serv.get_genes_by_organism(organism, start, max_results)
                self._buffer[organism] = {'genes':res, 'N': len(res)}
        else:
            res = self.serv.get_genes_by_organism(organism, start, max_results)
            self._buffer[organism] = {'genes':res, 'N': len(res)}
        return res


    def get_genes_by_pathway(self, pathway):
        self._check_pathway_id(pathway)
        if pathway.startswith("path:")==False:
            raise Exception("pathway must start with 'path:'")
        return self.serv.get_genes_by_pathway(pathway)



    def _get_Npathways(self):
        if self._pathways == None:
            org = self.pathways
        N = len([x['entry_id'] for x in self.pathways])
        return N
    Npathways = property(_get_Npathways, doc="nmber of pathways in the current orgamism (read-only)")


    def __str__(self):
        verbose = self.verbose
        self.verbose = False
        self.get_number_of_genes_by_organism(self.organism)
        txt = "Kegg database summary:\n"
        txt+= "  Number of organisms: %s\n"  % len(self.organisms.items)
        txt+= "  Number of pathways(%s): %s\n"  % (self.organism, self.Npathways)
        txt+= "  Number of genes(%s): %s\n"  % (self.organism, self._buffer[self.organism]['N'])

        for o in self._genes.keys():
            txt += " Number of genes (%s): %s" %(o, self._buffer[o]['N'])
        self.verbose = verbose
        return txt



    def get_reactions_by_pathway(self, pathway_id):
        """

        k.get_reactions_by_pathway('path:eco00260')
        k.get_reactions_by_pathway("path:hsa00010")

        """
        self._check_pathway_id(pathway_id)
        return self.serv.get_reactions_by_pathway(pathway_id)

    def get_kos_by_pathway(self, pathway_id):
        self._check_pathway_id(pathway_id)
        # Returns all ko_ids on the pathway map 'path:hsa00010'
        return self.serv.get_kos_by_pathway(pathway_id)



    def color_pathway_by_elements(self, pathway, element_id_list=[],
            fg_color_list=[], bg_color_list=[], show=True):
        """Show the pathway image in a browser with colored elements


        Color the elements (rectangles and circles on a pathway map)
        corresponding to the given **element_id_list** with the specified colors and
        return the URL of the colored image. 

        :param list element_id_list: list of elements ids to color (ids can be
            found from :meth:`get_elements_by_pathway`.
        :param list fg_color_list: the color of text and border of the elements
        :param list bg_color_list: the background color of the elements. 
        :param bool show: show the result in a browser

        This method is useful to specify which graphical object on the pathway to be
        colored as there are some cases that multiple genes are assigned to one
        rectangle or a gene is assigned to more than one rectangle on the pathway map.


        For more details on KGML, see: `<http://www.kegg.jp/kegg/soap/doc/keggapi_manual.html#label:33>`_
    
        ::

            # Returns the URL of the colored image of given pathway 'path:bsu00010' with
            # * gene bsu:BG11350 (element_id 78, ec:3.2.1.86) colored in red on yellow
            # * gene bsu:BG11203 (element_id 79, ec:3.2.1.86) colored in blue on yellow
            # * gene bsu:BG11685 (element_id 51, ec:2.7.1.2)  colored in red on orange
            # * gene bsu:BG11685 (element_id 47, ec:2.7.1.2)  colored in blue on orange
            element_id_list = [ 78, 79, 51, 47 ]
            fg_list  = [ '#ff0000', '#0000ff', '#ff0000', '#0000ff' ]
            bg_list  = [ '#ffff00', '#ffff00', '#ffcc00', '#ffcc00' ]
            k.color_pathway_by_elements('path:bsu00010', element_id_list, fg_list, bg_list)

        """
        assert len(fg_color_list)==len(element_id_list)
        assert len(bg_color_list)==len(element_id_list)

        res = self.serv.color_pathway_by_elements(pathway, element_id_list, fg_color_list, bg_color_list)
        print res
        if res and show==True:
            webbrowser.open(res)

    def color_pathway_by_objects(self, pathway, objects_id_list=[],
            fg_color_list=[], bg_color_list=[], show=True):
        """Show the pathway image in a browser with colored elements

        Same as :meth:`color_pathway_by_elements` but working with names
        instead of element_ids.

        """
        assert len(fg_color_list)==len(objects_id_list)
        assert len(bg_color_list)==len(objects_id_list)

        res = self.serv.color_pathway_by_objects(pathway,objects_id_list, fg_color_list, bg_color_list)
        print res
        if res and show==True:
            webbrowser.open(res)

    def mark_pathway_by_objects(self, pathway, objects_id_list=[], show=True):
        """Mark the given objects on the given pathway map and return the URL of the generated image."""
        res = self.serv.mark_pathway_by_objects(pathway,objects_id_list)
        print res
        if res and show==True:
            webbrowser.open(res)

    def get_html_of_marked_pathway_by_objects(self, pathway, objects_id_list=[],
            show=True):
        """HTML version of the :meth:`~pathways.kegg.Kegg.mark_pathway_by_objects` method. """
        res = self.serv.get_html_of_marked_pathway_by_objects(pathway,objects_id_list)
        print res
        if res and show==True:
            webbrowser.open(res)


    def get_html_of_colored_pathway_by_elements(self, pathway,
            element_id_list=[], fg_color_list=[], bg_color_list=[], show=True):
        """HTML version of the :meth:`~pathways.kegg.Kegg.color_pathway_by_elements` method. """
        assert len(fg_color_list)==len(element_id_list)
        assert len(bg_color_list)==len(element_id_list)

        res = self.serv.get_html_of_colored_pathway_by_elements(pathway, element_id_list, fg_color_list, bg_color_list)
        print res
        if res and show==True:
            webbrowser.open(res)

    def get_html_of_colored_pathway_by_objects(self, pathway,
            objects_id_list=[], fg_color_list=[], bg_color_list=[], show=True):
        """HTML version of the :meth:`~pathways.kegg.Kegg.color_pathway_by_objects` method. """
        assert len(fg_color_list)==len(objects_id_list)
        assert len(bg_color_list)==len(objects_id_list)

        res = self.serv.get_html_of_colored_pathway_by_objects(pathway, objects_id_list, fg_color_list, bg_color_list)
        if res and show==True:
            webbrowser.open(res)




    def get_references_by_pathway(self, pathway_id):
        """todo"""
        return self.serv.get_references_by_pathway(pathway_id)

    def view_reference(self, pathway_id):
        from browse.browser import browse
        browse("http://www.ncbi.nlm.nih.gov/pubmed/" + str(pathway_id))

    def view_all_references_by_pathway(self, pathway_id, maxtabs=10):
        """open all pubmed reference for a given pathway

        not in Kegg WSDL
        """
        refs = self.get_references_by_pathway(pathway_id)
        if len(refs)>maxtabs:
            print("%s references found. Openning only the first 10 tabs. change maxtabs if needed" % len(refs))
        for r in refs[0:min(maxtabs, len(refs))]:
            print r
            self.view_reference(r)


    def get_elements_by_pathway(self, pathway_id):
        elements = self.serv.get_elements_by_pathway(pathway_id)
        return elements

    def get_element_relations_by_pathway(self, pathway_id):
        """

        buffering
        """
        if self.organism in self._relations.keys():
            if pathway_id in self._relations[self.organism]:
                return self._relations[self.organism][pathway_id]
            else:
                relations = self.serv.get_element_relations_by_pathway(pathway_id)
                self._relations[self.organism][pathway_id] = relations
        else:
            self._relations[self.organism] = {}
            relations = self.serv.get_element_relations_by_pathway(pathway_id)
            self._relations[self.organism][pathway_id] = relations
        return self._relations[self.organism][pathway_id]




    def get_pathways_by_specy(self, specy):
        print "This will take a while since we are analysing all entries"
        results = self.lookfor(specy)
        print results

        pws = self.get_pathways_by_genes(gene)
        return pws
        

    def build_specy_ids_mapping(self, rebuild=False):
        """Build Mapping between genes IDs and thei description.

        :param bool rebuild: force the rebuild of the mapping. (default is False)

        Take a while. Saved in :attr:`_mapping` attribute.

        """
        print("Retrieving all genes for organism %s" % self.organism)
        genes = self.get_genes_by_organism()
        maxRequests = 100
        d = {}
        N = len(genes)
        calls = N/maxRequests

        entry = 0
        print("Building dictionary mapping Kegg id to description (names). Takes about 5 minutes depeding on organism")
        for i in range(0, calls):
            print("Analysing pathways %s/%s" % (i, calls))
            subgenes = genes[i*maxRequests:(i+1)*maxRequests]
            data = self.btit(" ".join([x for x in subgenes])).split("\n")
            for this in data:
                print this
                if len(this)>0:
                    keggid, description = this.split(" ", 1)
                    d[entry] = {'keggid':keggid, 'description': description}
                    entry+=1

        subgenes = genes[calls*maxRequests:]
        data = self.btit(" ".join([x for x in subgenes])).split("\n")
        for this in data:
            if len(this)>0:
                keggid, description = this.split(" ", 1)
                d[entry] = {'keggid':keggid, 'description': description}
                entry+=1

        self._mapping = d.copy()
        return self._mapping 

    def get_linked_pathways(self, pathway):
        return self.serv.get_linked_pathways(pathway)


    # SDDB related
    # =========================================
    def get_best_best_neighbors_by_gene(self, genes_id, offset=0, limit=10):
        """Search best-best neighbor of the gene in all organisms."""
        return self.serv.get_best_best_neighbors_by_gene(genes_id, offset, limit)

    def get_best_neighbors_by_gene(self, genes_id, offset=0, limit=10):
        """Search best neighbors in all organism."""
        return self.serv.get_best_neighbors_by_gene(genes_id, offset, limit)

    def get_reverse_best_neighbors_by_gene(self, genes_id, offset=0, limit=10):
        """Search reverse best neighbors in all organisms.

        ::

            >>> k.get_reverse_best_neighbors_by_gene('eco:b0002', 1, 10)

        """
        return self.serv.get_reverse_best_neighbors_by_gene(genes_id, offset, limit)

    def get_paralogs_by_gene(self, genes_id, offset=0, limit=10):
        """"Search paralogous genes of the given gene in the same organism."""
        return self.serv.get_paralogs_by_gene(genes_id, offset, limit)

    # KO related
    # =========================================
    def get_genes_by_ko(self, kid, org=None):
        """Returns all genes corresponding to this ko id (all organism)"""
        if org ==None:
            org = self.organism
            return self.serv.get_genes_by_ko(kid, org)
        else:
            return self.serv.get_genes_by_ko(kid)
    def get_ko_by_gene(self, gid):
        return self.serv.get_ko_by_gene(gid)
    def get_genes_by_ko_class(self, classid, org, offset, limit):
        return self.serv.get_genes_by_ko_class(classid)
    def get_ko_by_ko_class(self, classid, org, offset, limit):
        return self.serv.get_ko_by_ko_class(self, classid)


    # LIGAND related
    def search_drugs_by_name(self, drug):
        """Returns a list of drugs having the specified name.

        :param str drug: drug name
        :return: drug_id

        :: 

            k.search_drugs_by_name("tetracyclin")
        """
        return self.serv.search_drugs_by_name(drug)
    def search_drugs_by_mass(self, mass, error):
        return self.serv.search_drugs_by_mass(mass, error)

    # HERE BELOW ARE IMPROVED FUNCTIONS NEW TO KEGG WSDL INTERFACE


    
    def extra_get_elements_by_pathway_and_type(self, pathway_id, type="gene"):
        """Same as :meth:`~pathways.kegg.Kegg.get_elements_by_pathway` with type selection.

        :param pathway_id: valid pathway id
        :param str type: (Default gene)


        """
        self._check_pathway_id(pathway_id)
        elts_ids = self.get_elements_by_pathway(pathway_id)
        return [e for e in elts_ids if e['type']==type]

    def view_pathways(self, pathways=[], genes_ids=[], requires_all=False):
        """show a pathway in your browser.

        pattern can be a valid pathway id or a list of genes that must be
        contained in the pathway


        """

        # pathways must be a list but if only one pathway, users may provide a
        # string, so we need to convert it:
        if len(pathways) != 0:
            if isinstance(pathways, list) == False and isinstance(pathways, str):
                pathways = [pathways]

        if isinstance(genes_ids, list)==False:
            raise Exception("genes_ids must be a list")
        # look for pathways that contains genes_ids
        if len(pathways) == 0:
            for g in genes_ids:
                newpws = self.get_pathways_by_genes(g)

                if requires_all==False:
                    pathways.extend(newpws)
                else:
                    if len(pathways) == 0:
                        pathways = newpws
                    else:
                        newpws = set(newpws)
                        pathways = set(pathways).intersection(newpws)
            pathways = list(set(pathways))
            print "Found %s pathways" %len(pathways)
        else:
            pass


        print pathways

        for p in pathways:
            print "showin pathway", p
            element_ids = []
            for g in genes_ids:
                print " -- fetching id", p, g
                element = self._get_element_id_by_pathway(p, g)
                if element:
                    element_ids.append(element)
            print element_ids
            bg =["red"] * len(element_ids)
            fg =["white"] * len(element_ids)
            self.color_pathway_by_elements(p, element_ids, bg, fg)
            import time
            time.sleep(1)

    def _get_element_id_by_pathway(self, pathway_id, gene=None):
        elements = self.get_elements_by_pathway(pathway_id)
        if gene == None:
            return elements
        else:
            for e in elements:
                if gene in e['names']:
                    return e['element_id']

    def get_sif(self, pathway_id):
        """Export a pathway in SIF format

        .. note:: experimental, not to be used.


        """
        elements = self.extra_get_elements_by_pathway_and_type(pathway_id, type="gene")
        relations = self.get_element_relations_by_pathway(pathway_id)

        print "ok"
        for r in relations:
            print r
            link = r['subtypes'][0]
            print link
            (id1, id2) = r['element_id1'], r['element_id2']
            print("==========")
            #if id1 not in elements['element_id']:
            #    print "id1 %s not found" %id1
            #if id2 not in elements['element_id']:
            #    print "id2 %s not found" %id2

            print 'test'
            if link['relation'] == 'activation':
                print id1, 1, id2
            elif  link['relation'] == 'inibition':
                print id1, -1, id2
            else:
                print "#", id1, link['relation'], id2


    def load_mapping(self, filename=None):
        """Load a pickle of the buffered genes save by :meth:`~pathways.kegg.Kegg.save_kegg_names`

        :param str filename: a pickled file (can be zipped; see note below)

        Can be a zipped file but the zip archive must have been built as follows::

            zip archive.dat.zip archive.dat

        In other word, we expect to find a file inside the archive that has the
        name **archive.dat**

        """
        import pickle

        if filename.endswith("zip"):

            import zipfile, os
            # read the zip file given the full filename
            fh = zipfile.ZipFile(filename)
            # assuming the filename ending in zip is the same file that we want
            # to access to in the archive. First, get the filename without path
            filename = os.path.split(filename)[1]
            # second, get rid of extension
            filename = filename.split(".zip")[0]
            # Finally, extract the file
            data = fh.read(filename)
            # and since it should be a pickle, load it.
            self._mapping = pickle.loads(data)  # note use loads method NOT load 
        else:
            self._mapping = pickle.load(open(filename, "r"))

    def save_mapping(self, filename):
        """Save a pickle of the genes attribute in a file.

        """
        import pickle
        pickle.dump(self._mapping, open(filename, "w"))

    def lookfor_organism(self, query):
        return self.organisms.lookfor(query)

    def lookfor_specy(self, pattern):
        """Search for a specy in the gene database

        Specy are stored according to their Kegg id. In general, 
        one look for a specy given its usual name (e.g., LAT, ZAP70, ...)

        This function allows to search for a Kegg id given a name. 

        It takes a long time to search the database that way. So, we built a 
        dictionary that is stored within the package. 

        Otherwise, if you want to use the latests database, you can call::

            k.build_specy_ids_mapping()

        before hand.

        :return: list of tuples containing keggid and their corresponding
            description.
        """
        if self._mapping == None:
            import os
            filename = "kegg-hsa-gene_mapping_oct_2012.dat.zip"
            print "!! Reversed dictionary of names/genes is empty."
            print "\tLoading %s from the library" % filename
            print "\tYou can rebuild the dictionary (and save it) that way "
            print "\t >>> self.build_specy_ids_mapping()"
            print """\t >>> self.save_mapping("test.dat")"""
            print "\t and reload it before calling that function"
            print """\t >>> self.load_mapping("test.dat")"""
            from easydev import get_shared_directory_path
            db = get_shared_directory_path("pathways")
            self.load_mapping(db + os.sep + "data" +os.sep + filename)

        found = [v for k,v in self._mapping.iteritems() if pattern.lower() in v['description'].lower()]
        if self.verbose:
            print found
        return found


    def extra_get_all_relations(self, Nmax=None):
        """Returns all relations found in all pathways

        :param int Nmax: max number of pathways to look at.

        .. todo:: Takes a while to complete. Could save the results in a pickle.

        Output to be used by :meth:`~pathways.kegg.Kegg.extra_count_relations`

        """
        all_relations = []
        if Nmax == None:
            Nmax = len(self.pathways)
        pids = [p['entry_id'] for p in self.pathways[0:Nmax]]
        for i,pathway_id in enumerate(pids):
            print "processing %s/%s (%s)" % (i+1, Nmax, pathway_id)
            all_relations.append(self.get_element_relations_by_pathway(pathway_id))
        return all_relations


    def extra_count_relations(self, all_relations):
        """Returns number of relations in a dictionary 

        :param all_relations: output of :meth:`~pathways.kegg.Kegg.extra_get_all_relations`

        :return: dictionary of number of relations for each relation category.
        """
        counter = {}
        for relations in all_relations:
            these_relations = [r['subtypes'][0]['relation'] for r in relations if len(r['subtypes'])]
            for this in these_relations:
                if this not in counter.keys():
                    counter[this] = 1
                else:
                    counter[this] = counter[this] + 1
        return counter

    def info_database(self):
        """prints the databases definition and entry IDs

        .. seealso:: :meth:`list_databases`"""
        for x in self.list_databases:
            print("%50s: %s" %(  x['definition'], x['entry_id']))


    def extra_pathway_summary(self, pathway_id, show=True):
        """A tentative of pathway summary

        """
        assert pathway_id in [x['entry_id'] for x in self.pathways]
        # get genes of the pathway
        genes_ids = self.get_genes_by_pathway(pathway_id)
        # get information for each gene (e.g., name that appear in the image)
        names = []
        for i, gid in enumerate(genes_ids):
            print "analysing genes %s (%s out of %s)" % (gid, i, len(genes_ids))
            self.analyse(gid)
            #names.append(self.analyse(gid)['NAME']

        if show:
            self.view_pathways(pathway_id)

        # pathways contained in the pathway itself
        print "===========\nPathways contained in this pathway:\n\n"
        pids = [x for x in self.extra_get_elements_by_pathway_and_type(pathway_id, type="map")]
        for pathway_id in pids:
            results = self.lookfor_pathway(pathway_id['names'][0])
            for res in results:
                print(" *"+ res.definition+" "+ res.entry_id)

    def extra_get_pathway_types(self, pathway_id):
        """Return types found in a given pathway

        :param str pathway_id:


        """
        types = [x['type'] for x in self.get_elements_by_pathway(pathway_id)]
        return set(types)




    def extra_get_elements_by_pathway(self, pathway_id):
        """Returns elements id from a pathway (and more)

        :param str pathway_id:

        Given a pathway, this function returns a dictionary with the element_id,
        type, components and usual name of the elements found in the pathway.

        """
        elements = self.get_elements_by_pathway(pathway_id)
        d = {}
        for e in elements: 
            usual_names = []
            for name in e['names']:
                res = self.analyse(name)
                if 'NAME' in res.keys():
                    usual_names.append(res['NAME'])
            d[e['element_id']] = {'names': e['names'], 
                'type': e['type'],
                'components': e['components'],
                'usual_names': usual_names}
        return d


    def _extra_get_gene_element_id(self, pathway_id):
        """Given a pathway, returns elements type

        :param str pathway_id:
        """
        elements = self.get_elements_by_pathway(pathway_id)
        d = {}
        for e in elements: 
            if e['type'] == 'gene':
                for name in e['names']:
                    d[name] = e['element_id']
        return d



    def lookfor_pathway(self, query):
        """Search for a pathway in the databases

        :param str query: can be an id, a word

        case insensitive

        ::
    
            >>> k.lookfor_pathway("hsa:04604")

        """
        pathways = []
        for item in self.pathways:
            if query.lower() in item['definition'].lower():
                pathways.append(item)
            if query.lower() in item['entry_id'].lower():
                pathways.append(item)
        return pathways 






from bioservices import Kegg, KeggParser
from nose import with_setup
import unittest



# This is a simple test class that do not test everything on purpose.
# The other class could be use to test the code more thoroughly but it takes several
# minutes so during development this one should be used instead.
# class TestKeggAll should serve as a complement to this class
class TestKegg(object):

    @classmethod
    def setup_class(klass):
        klass.ws = Kegg()
        klass.ws.organismIds
        klass.ws.organism = "hsa"

    def test_isOrganism(self):
        assert self.ws.isOrganism('T01440') == True
        assert self.ws.isOrganism('hsa') == True
        assert self.ws.isOrganism('dummy') == False

    def test_database_IDs(self):
        self.ws.pathwayIds

    def test_conv(self):
        self.ws.conv("ncbi-gi","hsa:10458+ece:Z5100")

    def test_info(self):
        self.ws.info()
        self.ws.info("brite")

    def test_list(self):
        self.ws.list("pathway", "hsa")      # returns the list of human pathways

    def test_find(self):
        self.ws.find("compound", "300-310", "mol_weight")

    def test_get(self):
        self.ws.get("cpd:C01290+gl:G00092")

    def test_checkDB(self):
        for this in ["info", "list", "find", "link"]:
            try:
                self.ws._checkDB("dummy", this)
                assert False
            except:
                assert True
            self.ws._checkDB("pathway", this)

    def test_link(self):
        self.ws.link("pathway", "hsa:10458+ece:Z5100")

    def test_org_conv(self):
        assert 'hsa' == self.ws.Tnumber2code("T01001")
        assert 'T01001' == self.ws.code2Tnumber("hsa")

    def test_parse_kgml_pathway(self):
        res = self.ws.parse_kgml_pathway("hsa04660")





class _TestKeggALL(object):

    @classmethod
    def setup_class(klass):
        klass.ws = Kegg()
        klass.ws.organismIds
        print(klass)

    def test_ids(self):
        assert self.ws.enzymeIds[0].startswith("ec")
        assert self.ws.compoundIds[0].startswith("cpd")
        assert self.ws.glycanIds[0].startswith("gl")
        assert self.ws.reactionIds[0].startswith("rn")
        assert self.ws.drugIds[0].startswith("dr")
        assert self.ws.koIds[0].startswith("ko")
        assert self.ws.briteIds[0].startswith("br")


    def test_lookfor(self):
        self.ws.lookfor_organism("human")
        self.ws.lookfor_pathway("cell")

    def test_organism(self):
        self.ws.organism = "hsa"
        try:
            self.ws.organism = "dummy"
            assert False
        except:
            assert True

    def test_pathwayIDs(self):
        self.ws.organism = "hsa"
        self.ws.pathwayIds

    def test_info(self):
        self.ws.info("hsa")
        try:
            self.ws.info("dummy")
            assert False
        except:
            assert True

    def test_list(self):
        self.ws.list("pathway")             # returns the list of reference pathways
        self.ws.list("organism")            # returns the list of KEGG organisms with taxonomic classification
        self.ws.list("hsa")                 # returns the entire list of human genes
        self.ws.list("T01001")              # same as above
        self.ws.list("hsa:10458+ece:Z5100") # returns the list of a human gene and an E.coli O157 gene
        self.ws.list("cpd:C01290+gl:G00092")# returns the list of a compound entry and a glycan entry
        self.ws.list("C01290+G00092")       # same as above 

        # invalid queries:
        try:
            self.ws.list("drug", "hsa")
            assert False
        except:
            assert True

        try:
            self.ws.list("dumy")
            assert False
        except:
            assert True

    def test_find(self):
        self.ws.find("genes", "shiga+toxin")             # for keywords "shiga" and "toxin"
        self.ws.find("genes", "shiga toxin")            # for keywords "shigatoxin"
        self.ws.find("compound", "C7H10O5", "formula")   # for chemicalformula "C7H10O5"
        self.ws.find("compound", "O5C7","formula")       # for chemicalformula containing "O5" and "C7"
        self.ws.find("compound", "174.05","exact_mass")  # for 174.045 =<exact mass < 174.055
        self.ws.find("compound", "300-310","mol_weight") # for 300 =<molecular weight =< 310 

    def test_get(self):
        self.ws.get("C01290+G00092")
        self.ws.get("hsa:10458+ece:Z5100")
        self.ws.get("hsa:10458+ece:Z5100", "aaseq") 
        res = self.ws.get("hsa05130", "image")      
        try:
            self.ws.get("hsa05130", "imagffe")      
            assert False
        except:
            assert True

    def test_conv(self):
        self.ws.conv("ncbi-gi","hsa:10458+ece:Z5100")

        try:
            self.ws.conv("unipro", "hsa")
            assert False
        except:
            assert True

        try:
            self.ws.conv("uniprot", "hs")
            assert False
        except:
            assert True

        try:
            self.ws.conv("hs", "unipro")
            assert False
        except:
            assert True

        try:
            self.ws.conv("hsa", "unipr")
            assert False
        except:
            assert True

        # asc contains 1500. Try to get even samller to spped up tests.
        #self.conv("asc", "uniprot")
        self.ws.conv("hsa","up:Q9BV86+")

    def test_show_module(self):
        self.ws.show_module("md:hsa_M00402")


    def test_show_pathway(self):
        self.ws.show_entry("path:hsa05416")
        self.ws.show_pathway("path:hsa05416", scale=50)

    def test_check_dentries(self):
        assert True == self.ws.check_dbentries("hsa:10458+ece:Z5100")
        try:
            self.ws.check_dbentries("hsa:10458+ece:Z510", checkAll=False)
            assert False
        except:
            assert True

    def pathway2sif(self):
        sif = self.ws.pathway2sif("path:hsa05416")


#@unittest.skip
def test_KeggParser():
    s = KeggParser()
    d = s.parse(s.get("cpd:C00001"))
    d = s.parse(s.get("ds:H00001"))
    d = s.parse(s.get("dr:D00001"))
    d = s.parse(s.get("ev:E00001"))
    d = s.parse(s.get("ec:1.1.1.1"))
    d = s.parse(s.get("hsa:1525"))
    d = s.parse(s.get("genome:T00001"))
    d = s.parse(s.get("gl:G00001"))
    d = s.parse(s.get("md:hsa_M00554"))
    d = s.parse(s.get("ko:K00001"))
    d = s.parse(s.get("path:hsa04914"))
    d = s.parse(s.get("rc:RC00001"))
    d = s.parse(s.get("rn:R00001"))
    d = s.parse(s.get("rp:RP00001"))












from bioservices import Miriam




class test_miriam(Miriam):

    def __init__(self):
        super(test_miriam, self).__init__(verbose=False)


    def test_checkRegExp(self):
        assert self.checkRegExp("1P43403", "uniprot") == False
        assert self.checkRegExp("P43403", "uniprot") == True

    def test_convertURL(self):
        assert self.convertURL("http://identifiers.org/ec-code/1.1.1.1") == 'urn:miriam:ec-code:1.1.1.1'

    def test_convertURLs(self):
        self.convertURLs(['http://identifiers.org/pubmed/16333295', 'http://identifiers.org/ec-code/1.1.1.1'])

    def test_convertURN(self):
        res = self.convertURN('urn:miriam:ec-code:1.1.1.1')
        assert res == 'http://identifiers.org/ec-code/1.1.1.1'

    def test_convertURNs(self):
        res = self.convertURNs(['urn:miriam:ec-code:1.1.1.1'])
        assert res == ['http://identifiers.org/ec-code/1.1.1.1']

    def test_getDataResources(self):
        self.getDataResources("uniprot")

    def test_getDataTypeDef(self):
        self.getDataTypeDef("uniprot")

    def test_getDataTypePattern(self):
        self.getDataTypePattern("uniprot")

    def test_getDataTypeSynonyms(self):
        assert len(self.getDataTypeSynonyms("uniprot"))>0

    def test_getDataTypeURI(self):
        assert self.getDataTypeURI("uniprot") == 'urn:miriam:uniprot'

    def test_getDataTypeURIs(self):
        self.getDataTypeURIs("uniprot")
        ['urn:miriam:uniprot', 'urn:lsid:uniprot.org:uniprot', 'urn:lsid:uniprot.org', 'http://www.uniprot.org/']


    def test_getDataTypesId(self):
        self.getDataTypesId()

    def test_getDataTypesName(self):
        self.getDataTypesName()

    def test_getJavaLibraryVersion(self):
        self.getJavaLibraryVersion()

    def test_getLocation(self):
        self.getLocation("UniProt", "MIR:00100005")
        self.getLocations("urn:miriam:obo.go:GO%3A0045202")


    def test_getLocations(self ):
        self.getLocations("UniProt","P62158")
        #>>> m.serv.getLocations("urn:miriam:obo.go:GO%3A0045202")

    def test_getLocationsWithToken(self):
        self.getLocationsWithToken("uniprot", "P43403")

    def test_getMiriamURI(self):
        res = self.getMiriamURI("http://www.ebi.ac.uk/chebi/#CHEBI:17891")
        assert res == 'urn:miriam:chebi:CHEBI%3A17891'

    def test_getName(self):
        self.getName('uniprot')

    def test_getNames(self):
        self.getNames('uniprot')
    
    def test_getOfficialDataTypeURI(self):
        self.getOfficialDataTypeURI("chEBI")
        #'urn:miriam:chebi'

    def test_getResourceInfo(self):
        res = self.getResourceInfo("MIR:00100005")
        assert res == 'MIRIAM Resources (data collection)'


    def test_getResourceInstitution(self):
        res = self.getResourceInstitution("MIR:00100005")
        assert res == "European Bioinformatics Institute"

    def test_getResourceLocation(self):
        res = self.getResourceLocation("MIR:00100005")
        assert res == "United Kingdom"

    def test_getServicesInfo(self):
        self.getServicesInfo()

    def test_getServicesVersion(self):
        self.getServicesVersion()

    def test_getURI(self):
        res = self.getURI("UniProt", "P62158")
        assert res == 'urn:miriam:uniprot:P62158'

    def test_getURIs(self):
        res = self.getURIs("UniProt", "P62158")

    def test_isDeprecated(self):
        self.isDeprecated("urn:miriam:uniprot")
    




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: wsdbfetch.py 116 2013-01-31 10:45:56Z leamoerchharder $
"""Interface to WSDbfetch web service

.. topic:: What is WSDbfetch

    :URL: http://www.ebi.ac.uk/Tools/webservices/services/dbfetch
    :Service: http://www.ebi.ac.uk/Tools/webservices/services/dbfetch_rest

    .. highlights::

        "WSDbfetch allows you to retrieve entries from various up-to-date biological
        databases using entry identifiers or accession numbers. This is equivalent to
        the CGI based dbfetch service. Like the CGI service a request can return a
        maximum of 200 entries."

        -- From http://www.ebi.ac.uk/Tools/webservices/services/dbfetch , Dec 2012


"""
from services import WSDLService
import urllib2




class WSDbfetch(WSDLService):
    """Interface to `WSDbfetch <http://www.ebi.ac.uk/Tools/webservices/services/dbfetch_rest>`_ service

    ::

        >>> from bioservices import WSDbfetch
        >>> w = WSDbfetch()
        >>> data = w.fetchBatch("uniprot" ,"zap70_human", "xml", "raw")

    The actual URL used is http://www.ebi.ac.uk/ws/services/WSDbfetchDoclit?wsdl from
    biocatalogue (this one having let functionalities: 
    http://www.ebi.ac.uk/ws/services/WSDbfetch?wsdl).

    """
    _url = 'http://www.ebi.ac.uk/ws/services/WSDbfetchDoclit?wsdl'
    def __init__(self,  verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: print informative messages
        """
        super(WSDbfetch, self).__init__(name="WSDbfetch", url=WSDbfetch._url, 
            verbose=verbose)
        self._supportedDBs = None
        self._supportedFormats = None
        self._supportedStyles = None

    def _check_db(self, db):
        if db not in self.supportedDBs:
            raise Exception("%s not a supportedDB. " %db)


    def fetchBatch(self, db, ids, format="default", style="default"):
        """Fetch a set of entries in a defined format and style.

        :param str db: the name of the database to obtain the entries from (e.g. 'uniprotkb').
        :param list query: list of identifiers (e.g. 'wap_rat, wap_mouse').
        :param str format: the name of the format required. 
        :param str style: the name of the style required. 

        :returns: The format of the response depends on the interface to the service used:

            * WSDBFetchServerService and WSDBFetchDoclitServerService: The entries as a string.
            * WSDBFetchServerLegacyService: An array of strings containing the entries. 


        ::

            from bioservices import WSDbfetch
            u = WSDbfetch()
            u.fetchBatch("uniprot" ,"wap_mouse", "xml")

        """
        res = self.serv.fetchBatch(db, ids, format, style)
        res = self.easyXML(res)
        return res

    def fetchData(self, query, format="default", style="default"):
        """Fetch an entry in a defined format and style.

        :param str query: the entry identifier in db:id format (e.g. 'UniProtKB:WAP_RAT').
        :param str format: the name of the format required. 
        :param str style: the name of the style required. 

        :returns: The format of the response depends on the interface to the service used:

            * WSDBFetchServerService and WSDBFetchDoclitServerService: The entries as a string.
            * WSDBFetchServerLegacyService: An array of strings containing the entries. Generally 
              this will contain only one item which contains the set of entries.

        ::

            from bioservices import WSDbfetch
            u = WSDbfetch()
            u.fetchData('uniprot:zap70_human')

        """
        res = self.serv.fetchData(query, format, style)
        return res


    def getDatabaseInfo(self, db):
        """Get details describing specific database (data formats, styles)

        :param str db: a valid database. 
        :return: The output can be introspected and contains several attributes
            (e.g., displayName).

        :: 

            >>> res = u.getDatabaseInfo("uniprotkb")
            >>> res.displayName
            'UniProtKB'
            >>> print(res.description.encode('utf-8'))
            u'The UniProt Knowledgebase (UniProtKB) is the central access point for extensive curated protein information, including function, classification, and cross-references. Search UniProtKB to retrieve \u201ceverything that is known\u201d about a particular sequence.'

        """
        self._check_db(db)
        return self.serv.getDatabaseInfo(db)

    def getDatabaseInfoList(self):
        """Get details of all available databases, includes formats and result styles.

        :Returns: A list of data structures describing the databases. See
            :meth:`getDatabaseInfo` for a description of the data structure.
        """
        return self.serv.getDatabaseInfoList()

    def getDbFormats(self, db):
        """Get list of format names for a given database.


        :param str db:

        """
        self._check_db(db)
        return self.serv.getDbFormats(db)
    

    def getFormatStyles(self, db, format):
        """Get a list of style names available for a given database and format.

        :param str db: database name to get available styles for (e.g. uniprotkb).
        :param str format: the data format to get available styles for (e.g. fasta).

        :Returns: An array of strings containing the style names. 

        ::

            >>> u.getFormatStyles("uniprotkb", "fasta")
            ['default', 'raw', 'html']
        """

        self._check_db(db)
        return self.serv.getFormatStyles(db, format)

    def getSupportedDBs(self):
        """Get a list of database names usable with WSDbfetch. 

        Buffered in _supportedDB.
        """
        if self._supportedDBs:
            return self._supportedDBs
        else:
            self._supportedDBs = self.serv.getSupportedDBs()
        return self._supportedDBs

    supportedDBs = property(getSupportedDBs, doc="Alias to getSupportedDBs.")

    def getSupportedFormats(self):
        """Get a list of database and format names usable with WSDbfetch.

        .. deprecated:: use Of getDbFormats(db), getDatabaseInfo(db) or  getDatabaseInfoList().



        """
        if self._supportedFormats == None:
            self._supportedFormats = self.serv.getSupportedFormats()
        return self._supportedFormats
    supportedFormats = property(getSupportedFormats)

    def getSupportedStyles(self):
        """Get a list of database and style names usable with WSDbfetch.

        .. deprecated:: use Of getFormatStyles(db, format), getDatabaseInfo(db) or         getDatabaseInfoList() is recommended.

        Returns: An array of strings containing the database and style names. 
        """
        if self._supportedStyles == None:
            self._supportedStyles = self.serv.getSupportedStyles()
        return self._supportedStyles
    supportedStyles = property(getSupportedStyles)







#from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

import services
from services import *

import biomodels
from biomodels import *

import chebi
from chebi import *

import kegg
from kegg import *

import rhea
from rhea import *

import xmltools
from xmltools import *

import wikipathway
from wikipathway import *

import pdb
from pdb import *

import uniprot
from uniprot import *

import unichem
from unichem import *

import wsdbfetch
from wsdbfetch import *

import xmltools
from xmltools import *

import reactome
from reactome import *

import quickgo
from quickgo import *

import chembldb
from chembldb import *

import picr
from picr import *

import psicquic
from psicquic import *

import ncbiblast
from ncbiblast import *

import biogrid
from biogrid import *

import miriam
from miriam import *

import arrayexpress
from arrayexpress import *

import biomart
from biomart import *

import eutils
from eutils import *



from bioservices import UniChem




def test_unichem_src_compound():
    uni = UniChem()
    uni.get_src_compound_ids_from_src_compound_id("CHEMBL12", "chembl", "chebi")
    uni.get_src_compound_ids_all_from_src_compound_id("CHEMBL12", "chembl","drugbank")
    #[{'assignment': '1', 'src_compound_id': 'DB00829'},
    #{'assignment': '0', 'src_compound_id': 'DB07699'}]


def test_unichem_src_compound_from_inchikey():
    uni = UniChem()
    uni.get_src_compound_ids_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")
    uni.get_src_compound_ids_all_from_inchikey("AAOVKJBEBIDNHE-UHFFFAOYSA-N")


def test_mapping():
    uni = UniChem()
    res1 = uni.get_mapping("kegg_ligand", "chembl")
    assert len(res1)>0

def test_src_ids():
    uni = UniChem()
    uni.get_all_src_ids()
    uni.get_source_information("chembl")


def test_structure():
    uni = UniChem()
    uni.get_structure("CHEMBL12", "chembl")
    uni.get_structure_all("CHEMBL12", "chembl")


def test_get_source_id():
    uni = UniChem()
    assert uni.get_source_id("chembl") == 1
    assert uni.get_source_id("1") == 1
    assert uni.get_source_id(1) == 1

    try:
        uni.get_source_id("wrong")
        assert False
    except:
        assert True

    try:
        uni.get_source_id("20000")
        assert False
    except:
        assert True

def test_get_src_compound_ids_all_from_obsolete():
    uni = UniChem()
    res = uni.get_src_compound_ids_all_from_obsolete("DB07699", 2)
    res = uni.get_src_compound_ids_all_from_obsolete("DB07699", 2, "chembl")

def test_get_verbose_src_compound_ids_fron_inchikey():
    uni = UniChem()
    uni.get_verbose_src_compound_ids_from_inchikey("GZUITABIAKMVPG-UHFFFAOYSA-N")


def test_get_src_compoundid_url():
    uni = UniChem()
    uni.get_src_compound_id_url("CHEMBL12", "chembl", "drugbank")


def test_get_auxiliary_mapping():
    # this does nothing (behaviour of the function)
    uni = UniChem()
    res = uni.get_auxiliary_mappings(1)
    # this returns something but takes lots of time so it is commented
    #res = uni.get_auxiliary_mappings(15)




# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: kegg.py 220 2013-06-13 16:47:12Z cokelaer $
"""This module provides a class :class:`~Kegg` to access to the
REST Kegg interface. There are additional methods and functionalities added by 
**BioServices**.

.. note:: a previous imterface to the KEGG WSDL service was designed but the
    WSDL closed in  Dec 2012.

.. topic:: What is KEGG ?

    :URL: http://www.kegg.jp/
    :REST: http://www.kegg.jp/kegg/rest/keggapi.html
    :weblink: http://www.genome.jp/kegg/rest/weblink.html
    :dbentries: http://www.genome.jp/kegg/rest/dbentry.html

    .. highlights::

        "KEGG is a database resource for understanding high-level functions and
        utilities of the biological system, such as the cell, the organism and the
        ecosystem, from molecular-level information, especially large-scale molecular
        datasets generated by genome sequencing and other high-throughput experimental
        technologies (See Release notes for new and updated features). "

        -- Kegg home page, Jan 2013

.. _terminology:

Some terminology
--------------------

The following list is a simplified list of terminology taken from Kegg API
pages.


* organisms (**org**) are made of a three-letter (or four-letter) code (e.g.,
  **hsa** stands for Human Sapiens) used in KEGG (see  :attr:`~bioservices.kegg.Kegg.organismIds`). 
* **db** is a database name. See :attr:`~bioservices.kegg.Kegg.databases`
  attribute and :ref:`kegg_database` section.
* **entry_id** is a unique identifier. It is a combination of the database name
  and the identifier of an entry joined by a colon sign (e.g. 'embl:J00231').

  Here are some examples of entry Ids:

    * **genes_id**: A KEGG organism and a gene name (e.g. 'eco:b0001').
    * **enzyme_id**: 'ec' and an enzyme code. (e.g. 'ec:1.1.1.1'). See :attr:`~bioservices.kegg.Kegg.enzymeIds`.
    * **compound_id**: 'cpd' and a compound number (e.g. 'cpd:C00158'). 
      Some compounds also have 'glycan_id' and
      both IDs are accepted and converted internally.
      See :attr:`~bioservices.kegg.Kegg.compoundIds`.
    * **drug_id**: 'dr' and a drug number (e.g. 'dr:D00201'). See
      :attr:`~bioservices.kegg.Kegg.drugIds`.
    * **glycan_id**: 'gl' and a glycan number (e.g.
    * 'gl:G00050'). Some glycans also have 'compound_id' and both
      IDs are accepted and converted internally. see
      :attr:`~bioservices.kegg.Kegg.glycanIds` attribute.
    * **reaction_id**:  'rn' and a reaction number (e.g.
    * 'rn:R00959' is a reaction which catalyze cpd:C00103 into cpd:C00668).
      See :attr:`~bioservices.kegg.Kegg.reactionIds` attribute.
    * **pathway_id**: 'path' and a pathway number. Pathway numbers prefixed 
      by 'map' specify the reference pathway and pathways prefixed by 
      a KEGG organism specify pathways specific to the organism (e.g. 
      'path:map00020', 'path:eco00020'). See :attr:`~bioservices.kegg.Kegg.pathwayIds` attribute.
    * **motif_id**: a motif database names ('ps' for prosite, 'bl' for blocks, 
      'pr' for prints, 'pd' for prodom, and 'pf' for pfam) and a motif entry
      name. (e.g. 'pf:DnaJ' means a Pfam  database entry 'DnaJ').
    * **ko_id**: identifier made of 'ko' and a ko number (e.g. 'ko:K02598').
      See :attr:`~bioservices.kegg.Kegg.koIds` attribute.


.. _kegg_database:

Kegg Databases Names and Abbreviations
-------------------------------------------

Here is a list of databases used in KEGG API with their name and abbreviation:

=============== =========== ==========
Database Name   Abbrev      kid       
=============== =========== ==========
pathway         path        map number  
brite           br          br number   
module          md          M number    
disease         ds          H number  
drug            dr          D number  
environ         ev          E number 
orthology       ko          K number    
genome          genome      T number    
genomes         gn          T number
genes           -           -      
ligand          ligand      -     
compound        cpd         C number 
glycan          gl          G number    
reaction        rn          R number    
rpair           rp          RP number   
rclass          rc          RC number   
enzyme          ec          -   
=============== =========== ========== 


.. _db_entries:

Database Entries
-------------------

Database entries can be written in on of the following ways::

    <dbentries> = <dbentry>1[+<dbentry>2...]
    <dbentry> = <db:entry> | <kid> | <org:gene>

Each database entry is identified by::

    db:entry 

where "db" is the database name or its abbreviation shown above and  "entry" is the entry name or the accession number that is uniquely assigned within the database. In reality "db" may be omitted, for the entry name called the KEGG object identifier (kid) is unique across KEGG.::

    kid = database-dependent prefix + five-digit number

In the KEGG GENES database the db:entry combination must be specified. This is
more specifically written as::

    org:gene 

where "org" is the three- or four-letter KEGG organism code or the T number genome
identifier and "gene" is the gene identifier, usually locus_tag or ncbi GeneID, or the primary
gene name.


"""
from __future__ import print_function

from bioservices.services import RESTService, BioServicesError
#from services import RESTService, BioServicesError
import webbrowser
import copy


__all__ = ["Kegg", "KeggParser"]

class Kegg(RESTService):
    """Interface to the `KEGG <http://www.genome.jp/kegg/pathway.html>`_ service

    This class provides an interface to the KEGG REST API. The weblink tools 
    are partially accesible. All dbentries can be parsed into dictionaries using
    the :class:`KeggParser`

    Here are some examples. In order to retrieve the entry of the gene identifier 7535 of the 
    **hsa** organism, type::

        from bioservices import Kegg
        s = Kegg()
        print s.get("hsa:7535")
   
    The output is the raw ouput sent by KEGG API. See :class:`KeggParser` to 
    parse this output.
 
    .. seealso:: The :ref:`db_entries` to know more about the db entries format.
 
    Another example here below shows how to print the list of pathways of 
    the human organism::

        print s.list("pathway", organism="hsa")

    Further post processing would allow you to retrieve the pathway Ids. However,
    we provide additional functions to the KEGG API so the previous code and post 
    processing to extract the pathway Ids can be written as::

        s.organism = "hsa"
        s.pathwayIds

    and similarly you can get all :meth:`databases` output and database Ids easily. 
    For example, for the reaction database::

        s.reaction   # equivalent to s.list("reaction")
        s.reactionIds

    Other methods of interest are :meth:`conv`, :meth:`find`, :meth:`get`. We also provide 
    a :meth:`check_dbentries` to check validity of an entry Id.

    .. seealso:: :ref:`kegg_database`, :ref:`db_entries`, :ref:`terminology`.

    """

    #: valid databases
    _valid_DB_base = ["module", "disease", "drug","environ", "ko", 
        "genome", "compound", "glycan", "reaction", "rpair", "rclass", 
        "enzyme"]
    _valid_DB = _valid_DB_base + ["pathway", "brite", "genes", "ligand", \
        "organism", "genomes", "orthology"]
    _valid_databases_info = _valid_DB_base + ["pathway", "brite", "genes",\
        "ligand", "genomes", "kegg"]
    _valid_databases_list = _valid_DB_base + ["pathway", "brite", "organism"]
    _valid_databases_find = _valid_DB_base + ["pathway", "genes", "ligand"] 
    _valid_databases_link = _valid_DB_base + ["pathway", "brite"] 

    _docIds = "\n\n.. seealso:: :meth:`list`\n"
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: prints informative messages

        """
        super(Kegg, self).__init__(name="Kegg", url="http://rest.kegg.jp", verbose=verbose)
        self.easyXMLConversion = False
        self._organism = None

        self._organisms = None
        self._organisms_tnumbers = None
        self._pathway = None
        self._glycan = None
        self._compound = None
        self._ko = None
        self._enzyme = None
        self._reaction = None
        self._drug = None
        self._brite = None

        self._buffer = {}

        #self.keggParser = KeggParser()


    # we could use this to retrieve all databases Ids but 
    def __getattr__(self, req):
        """ for x in s._valid_databases_list: print len(getattr(s, x))"""
        if req.endswith("Ids"):
            db = req[0:-3]
            res = self.list(db)
            if db in ["",""]:
                Ids = [x.split()[1] for x in res.split("\n") if len(x)]
            else:
                Ids = [x.split()[0] for x in res.split("\n") if len(x)]
            return Ids
        elif req in self.databases:
            res = self.list(req)
            return res

    def code2Tnumber(self, code):
        """Converts organism code to its T number

        .. doctest::

            >>> from bioservices import *
            >>> s = Kegg()
            >>> s.code2Tnumber("hsa")
            'T01001'
        """
        index = self.organismIds.index(code)
        return self.organismTnumbers[index]

    def Tnumber2code(self, Tnumber):
        """Converts organism T number to its code

        .. doctest::

            >>> from bioservices import *
            >>> s = Kegg()
            >>> s.Tnumber2code("T01001")
            'hsa'
        """
        index = self.organismTnumbers.index(Tnumber)
        return self.organismIds[index]

    def isOrganism(self, org):
        """Checks if org is a KEGG organism

        :param str org:
        :return: True if org is in the KEGG organism list (code or Tnumber)

        ::

            >>> s.isOrganism("hsa")
            True

        """
        if org in self.organismIds:
            return True
        if org in self.organismTnumbers:
            return True
        else:
            return False

    def _checkDB(self, database=None, mode=None):
        self.logging.info("checking database %s (mode=%s)" % (database, mode))
        isOrg = self.isOrganism(database)

        if mode == "info":
            if database not in Kegg._valid_databases_info and isOrg == False:
                self.logging.error("database or organism provided is not correct (mode=info)")
                raise
        elif mode == "list":
            if database not in Kegg._valid_databases_list and isOrg == False:
                self.logging.error("database provided is not correct (mode=list)")
                raise 
        elif mode == "find":
            if database not in Kegg._valid_databases_find and isOrg == False:
                self.logging.error("database provided is not correct (mode=find)")
                raise 
        elif mode == "link":
            if database not in Kegg._valid_databases_link and isOrg == False:
                self.logging.error("database provided is not correct (mode=link)")
                raise 

        else:
            raise ValueError("mode can be only info, list, ")

    def info(self, database="kegg"):
        """Displays the current statistics of a given database

        :param str database: can be one of: kegg (default), brite, module,
            disease, drug, environ, ko, genome, compound, glycan, reaction,
            rpair, rclass, enzyme, genomes, genes, ligand or any 
            :attr:`organismIds`. 

        ::

            from bioservices import Kegg
            s = Kegg()
            s.info("hsa") # human organism
            s.info("T01001") # same as above
            s.info("pathway")

        """
        url = self.url+"/"+"info"
        self._checkDB(database, mode="info")

        url = url + "/" + database
        res = self.request(url)
        return res

    def list(self, query, organism=None):
        """Returns a list of entry identifiers and associated definition for a given database or a given set of database entries 

        :param str query: can be one of pathway, brite, module,
            disease, drug, environ, ko, genome, compound,
            glycan, reaction, rpair, rclass, enzyme, organism
            **or** an organism from the :attr:`organismIds` attribute **or** a valid
            dbentry (see below). If a dbentry query is provided, organism
            should not be used!
        :param str organism: a valid organism identifier that can be 
            provided. If so, database can be only "pathway" or "module". If 
            not provided, the default value is chosen (:attr:`organism`)
        :return: A string with a structure that depends on the query



        Here is an example that shows how to extract the pathways IDs related to
        the hsa organism::

            >>> s = Kegg()
            >>> res = s.list("pathway", organism="hsa")
            >>> pathways = [x.split()[0] for x in res.strip().split("\\n")]
            >>> len(pathways)  # as of Dec 2012
            261

        Note, however, that there are convenient aliases to some of the databases. 
        For instance, the pathway Ids can also be retrieved as a list from the
        :attr:`pathwayIds` attribute (after defining the :attr:`organism` attribute). 

        .. note:: If you set the query to a valid organism, then the second
               argument rganism is irrelevant and ignored.

        .. note:: If the query is not a database or an organism, it is supposed
            to be a valid dbentries string and the maximum number of entries is 100.

        Other examples::

            s.list("pathway")             # returns the list of reference pathways
            s.list("pathway", "hsa")      # returns the list of human pathways
            s.list("organism")            # returns the list of KEGG organisms with taxonomic classification
            s.list("hsa")                 # returns the entire list of human genes
            s.list("T01001")              # same as above
            s.list("hsa:10458+ece:Z5100") # returns the list of a human gene and an E.coli O157 gene
            s.list("cpd:C01290+gl:G00092")# returns the list of a compound entry and a glycan entry
            s.list("C01290+G00092")       # same as above 
        """
        url = self.url+"/"+"list"
        if query:
            #can be something else than a database so we can not use checkDB
            #self._checkDB(database, mode="list")
            url = url + "/" + query

        if organism:
            if organism not in self.organismIds:
                self.logging.error("""Invalid organism provided (%s). See the organismIds attribute""" % organism)
                raise BioServicesError("Not a valid organism")
            if query not in ["pathway", "module"]:
                self.logging.error("""
    If organism is set, then the first argument
    (database) must be either 'pathway' or 'module'. You provided %s""" % query)
                raise
            url = url + "/" + organism


        res = self.request(url)
        return res

    def find(self, database, query, option=None):
        """finds entries with matching query keywords or other query data in a given database 

        :param str database: can be one of pathway, module, disease, drug,
            environ, ko, genome, compound, glycan, reaction, rpair, rclass, 
            enzyme, genes, ligand or an organism code (see :attr:`organismIds`
            attributes) or T number (see :attr:`organismTnumbers` attribute).
        :param str query: See examples
        :param str option: If option provided, database can be only 'compound' 
            or 'drug'. Option can be 'formula', 'exact_mass' or 'mol_weight'


        .. note:: Keyword search against brite is not supported. Use /list/brite to 
            retrieve a short list.

        ::

            # search for pathways that contain Viral in the definition
            s.find("pathway", "Viral")
            # for keywords "shiga" and "toxin"
            s.find("genes", "shiga+toxin")
            # for keywords "shiga toxin"
            s.find("genes", ""shiga toxin")
            # for chemical formula "C7H10O5"
            s.find("compound", "C7H10O5", "formula")
            # for chemical formula containing "O5" and "C7"
            s.find("compound", "O5C7","formula")
            # for 174.045 =< exact mass < 174.055
            s.find("compound", "174.05","exact_mass")
            # for 300 =< molecular weight =< 310
            s.find("compound", "300-310","mol_weight")

        """
        _valid_options = ["formula", "exact_mass", "mol_weight"]
        _valid_db_options = ["compound", "drug"]

        self._checkDB(database, mode="find") 
        url = self.url + "/find/"+ database + "/" +  query

        if option:
            if database not in _valid_db_options:
                raise ValueError("invalid database. Since option was provided, database must be in %s" % _valid_db_options)
            if option not in _valid_options:
                raise ValueError("invalid option. Must be in %s " % _valid_options)
            url +=  "/" + option

        res = self.request(url)
        return res

    def show_entry(self, entry):
        """Opens URL corresponding to a valid entry

        ::

            s.www_bget("path:hsa05416")

        """
        url = "http://www.kegg.jp/dbget-bin/www_bget?" + entry
        self.logging.info(url)
        import webbrowser
        webbrowser.open(url)

    def get(self, dbentries, option=None):
        """Retrieves given database entries 

        :param str dbentries: KEGG database entries involving the following 
            database: pathway, brite, module, disease, drug, environ, ko, genome
            compound, glycan,  reaction, rpair, rclass, enzyme **or** any organism 
            using the KEGG organism code (see :attr:`organismIds`
            attributes) or T number (see :attr:`organismTnumbers` attribute).
        :param str option: one of: aaseq, ntseq, mol, kcf, image, kgml

      .. note:: you can add the option at the end of dbentries in which case
            the parameter option must not be used (see example)

        ::

            from bioservices import Kegg
            s = Kegg()
            # retrieves a compound entry and a glycan entry
            s.get("cpd:C01290+gl:G00092") 
            # same as above
            s.get("C01290+G00092")  
            # retrieves a human gene entry and an E.coli O157 gene entry
            s.get("hsa:10458+ece:Z5100")  
            # retrieves amino acid sequences of a human gene and an E.coli O157 gene
            s.get("hsa:10458+ece:Z5100/aaseq") 
            # retrieves the image file of a pathway map 
            s.get("hsa05130/image") 
            # same as above
            s.get("hsa05130", "image")


        Another example here below shows how to save the image of a given pathway::

           res =  s.get("hsa05130/image")
            # same as : res =  s.get("hsa05130","image")
            f = open("test.png", "w")
            f.write(res)
            f.close()

        .. note::  The input is limited up to 10 entries (KEGG restriction). 
        """
        _valid_options = ["aaseq", "ntseq", "mol", "kcf", "image", "kgml"]
        _valid_db_options = ["compound", "drug"]

        #self._checkDB(database, mode="find") 
        url = self.url + "/get/"+ dbentries

        if option:
            if option not in _valid_options:
                raise ValueError("invalid option. Must be in %s " % _valid_options)
            url +=  "/" + option

        res = self.request(url)

        return res


    def conv(self, target, source):
        """convert KEGG identifiers to/from outside identifiers 

        :param str target: the target database (e.g., a KEGG organism).
        :param str source: the source database (e.g., uniprot) or a valid
            dbentries; see below for details.

        :return: a dictionary with keys being the source and values being the target.

        Here are the rules to set the target and source parameters.

        If the second argument is not a **dbentries**, source and target
        parameters can be of two types:

            #. gene identifiers. If the target is a KEGG Id, then the source 
               must be one of *ncbi-gi*, *ncbi-geneid* or *uniprot*.

               .. note:: source and target can be swapped.
            #. chemical substance identifiers. If the target is one of the 
               following kegg database: drug, compound, glycan then the source 
               must be one of *pubchem* or *chebi*.

               .. note:: again, source and target can be swapped

        If the second argument is a **dbentries**, it can be again of two types:

            #. gene identifiers. The database used can be one ncbi-gi,
               ncbi-geneid, uniprot or any KEGG organism
            #. chemical substance identifiers. The database used can be one of
               drug, compound, glycan, pubchem or chebi only.

        .. note:: if the second argument is a dbentries, target and dbentries
            cannot be swapped.

        ::

            # conversion from NCBI GeneID to KEGG ID for E. coli genes
            conv("eco","ncbi-geneid")
            # inverse of the above example
            conv("eco","ncbi-geneid")
            #conversion from KEGG ID to NCBI GI 
            conv("ncbi-gi","hsa:10458+ece:Z5100")


        To make it clear by taking another example, you can either convert an
        entire database to another (e.g., from uniprot to Kegg Id all human gene
        IDs):: 

            uniprot_ids, kegg_ids = s.conv("hsa", "uniprot")

        or a subset by providing a valid **dbentries**::

            s.conv("hsa","up:Q9BV86+")


        .. warning:: dbentries are not check and are supposed to be correct. 
            See :meth:`check_idbentries` to help you checking a dbentries.

        .. versionchanged:: 1.1
            the output is now a dictionary, not a list of tuples
        """

        # The second argument may be a source_db or a dbentries so checking
        # second argument is kind of tricky because dbentries take lots of
        # different form.

        # for now, we only check the first argument.
        # gene identifiers
        isOrg = self.isOrganism(target)
        if isOrg==False and target not in ['ncbi-gi', 'ncbi-geneid', 'uniprot', 'pubchem',
                'chebi', 'drug', 'compound', 'glycan']:
                raise ValueError("""
    Invalid syntax. target must be a KEGG ID or
    one of the allowed database. See documentation og :meth:`conv` for
    details""")

        """if target in self.organismIds:
            if source not in ['ncbi-gi', 'ncbi-geneid', 'uniprot']:
                raise ValueError("Invalid source (must be ncbi-gi, ncbi-geneid or uniprot)")
        elif target in ['ncbi-gi', 'ncbi-geneid', 'uniprot']:
            if source not in self.organismIds:
                raise ValueError("Invalid source (must be a valid KEGG organism)")

        # for chenical substance identifiers
        elif target in ['drug', 'compound', 'glycan']:
            if source not in ['chebi', 'pubchem']:
                raise ValueError("Invalid source. Must be chebi or pubchem")
        elif target in ['chebi', 'pubchem']:
            if source not in ['drug', 'compound', 'glycan']:
                raise ValueError("Invalid source. Must be drug, compound or glycan")
        else:
            self.logging.info("arguments not checked")
        """


        url = self.url + "/conv/"+ target + '/' + source

        res = self.request(url)

        try:
            t = [x.split("\t")[0] for x in res.strip().split("\n")]
            s = [x.split("\t")[1] for x in res.strip().split("\n")]
            return dict([(x,y) for x,y in zip(t, s)])
        except:
            return res


    def link(self, target, source):
        """Find related entries by using database cross-references 

        :param str target: the target KEGG database or organism (see below for the list).
        :param str source: the source KEGG database or organism (see below for
            the list) or a valid dbentries involving one of the database; see
            below for details.

        The valid list of databases is pathway, brite, module, disease, drug, environ,
        ko, genome, compound, glycan, reaction, rpair, rclass, enzyme

        ::

            # KEGG pathways linked from each of the human genes
            s.link("pathway", "hsa") 
            # human genes linked from each of the KEGG pathways
            s.link("hsa", "pathway") 
            # KEGG pathways linked from a human gene and an E. coli O157 gene. 
            s.link("pathway", "hsa:10458+ece:Z5100")   
        """

        self._checkDB(target, mode="link") 

        url = self.url + "/link/"+ target + '/' + source
        res = self.request(url)
        return res

    def entry(self, dbentries):
        """Retrieve entry

        There is a weblink service (see http://www.genome.jp/kegg/rest/weblink.html)
        Since it is equivalent to :meth:`get`, we do not implement it for now 

        """
        raise NotImplementedError("Use :meth:`get` instead") 


    def show_pathway(self, pathId, scale=None, dcolor="pink", keggid={}):
        """Show a given pathway inside a web browser

        :param str pathId: a valid pathway Id. See :meth:`pathwayIds`
        :param int scale: you can scale the image with a value between 0 and 100
        :param str dcolor: set the default background color of nodes
        :param dict keggid: set color of entries contained in the pathway as 
            key/value pairs; can also be a list, in which case all nodes have 
            the same default color (red)

        .. note:: if scale is provided, dcolor and keggid are ignored.

        ::

            # show a pathway in the browser
            s.show_pathway("path:hsa05416", scale=50)

            # Same as above but also highlights some KEGG Ids (red for all)
            s.show_pathway("path:hsa05416", dcolor="white", 
                keggid=['1525', '1604', '2534'])

            # You can refine the colors using a dictionary:
            s.show_pathway("path:hsa05416", dcolor="white", 
                keggid={'1525':'yellow,red', '1604':'blue,green', '2534':"blue"})


        .. anotherway to higlight: http://www.kegg.jp/pathway/map00010+K00873+C00022
        """
        if pathId.startswith("path:"):
            pathId = pathId.split(":")[1]



        if scale:
            scale = int(scale/100.*100)/100. # just need 2 digits and a value in [0,1]
            url = "http://www.kegg.jp/kegg-bin/show_pathway?scale=" + str(scale)
            url += "&query=&map=" + pathId 
        else:
            url = "http://www.kegg.jp/kegg-bin/show_pathway?" + pathId
            if dcolor:
                url += "/default%%3d%s/" % dcolor
            if isinstance(keggid, dict):
                if len(keggid.keys())>0:
                    for k,v in keggid.iteritems():
                        if "," in v:
                            url += "/%s%%09%s/" % (k,v)
                        else:
                            url += "/%s%%09,%s/" % (k,v)
            elif isinstance(keggid, list):
                for k in keggid:
                    url += "/%s%%09,%s/" % (k,"red")

        self.logging.info(url)
        import webbrowser
        res = webbrowser.open(url)
        return res

    def show_module(self, modId):
        """Show a given module inside a web browser

        :param str modId: a valid module Id. See :meth:`moduleIds`

        Validity of modId is not checked but if wrong the URL will not open a 
        proper web page.
        """
        if modId.startswith("md:"):
            modId = modId.split(":")[1]
        url = "http://www.kegg.jp/module/" + modId
        self.logging.info(url)
        import webbrowser
        res = webbrowser.open(url)
        return res


    def check_dbentries(self, dbentries, checkAll=True):
        """Checks that all entries provided exist in the KEGG database

        :param str dbentries: a dbentries list. entries are separated by the +'
            sign (e.g., "hsa:10458+ece:Z5100")
        :param bool checkAll: checks all entries (Default) or stop as soon as an
            entry is not well formed.
        :return: True if all entries are correct. False otherwise

        ::

            s = Kegg()
            s.check_dbentries("hsa:10458+ece:Z5100")

        
        """
        import urllib2
        entries = dbentries.split("+")
        # we do not want logging here
        debugLevel = self.debugLevel
        self.debugLevel = "CRITICAL"
        allStatus = True
        for entry in entries:
            try:
                self.get(entry)
                status = True
            # if ill-formed, an entry will raise the 404 error.
            except urllib2.HTTPError, e:
                if e.code == 404:
                    status = False
                    allStatus = False
                    if checkAll == False:
                        print(entry, status)
                        return False
                else:
                    print(e)
                    raise
            except:
                self.debugLevel = debugLevel
                raise
            print(entry, status)
        # retrieve logging level
        self.debugLevel = debugLevel
        return allStatus

    # wrapper of all databases to ease access to them (buffered)

    def _get_db(self):
        return Kegg._valid_DB
    databases = property(_get_db, doc="Returns list of valid Kegg databases.")

    def _get_database(self, dbname, mode=0):
        res = self.list(dbname)
        assert mode in [0,1]
        return [x.split()[mode] for x in res.split("\n") if len(x)]

    def _get_organisms(self):
        if self._organisms == None:
            self._organisms = self._get_database("organism", 1)
        return self._organisms
    organismIds = property(_get_organisms, doc="Returns list of organism Ids")

    def _get_reactions(self):
        if self._reaction == None:
            self._reaction = self._get_database("reaction", 0)
        return self._reaction
    reactionIds = property(_get_reactions, doc="returns list of reaction Ids")

    def _get_enzyme(self):
        if self._enzyme == None:
            self._enzyme = self._get_database("enzyme", 0)
        return self._enzyme
    enzymeIds = property(_get_enzyme, 
        doc="returns list of enzyme Ids" + _docIds)

    def _get_organisms_tnumbers(self):
        if self._organisms_tnumbers == None:
            self._organisms_tnumbers = self._get_database("organism", 0)
        return self._organisms_tnumbers
    organismTnumbers = property(_get_organisms_tnumbers, 
        doc="returns list of organisms (T numbers)" + _docIds)

    def _get_glycans(self):
        if self._glycan == None:
            self._glycan = self._get_database("glycan", 0)
        return self._glycan
    glycanIds = property(_get_glycans, 
        doc="Returns list of glycan Ids" + _docIds)

    def _get_brite(self):
        if self._brite == None:
            self._brite = self._get_database("brite", 0)
        return self._brite
    briteIds = property(_get_brite, 
        doc="returns list of brite Ids." + _docIds)

    def _get_kos(self):
        if self._ko == None:
            self._ko = self._get_database("ko", 0)
        return self._ko
    koIds = property(_get_kos, doc="returns list of ko Ids" + _docIds)

    def _get_compound(self):
        if self._compound == None:
            self._compound =  self._get_database("compound", 0)
        return self._compound
    compoundIds = property(_get_compound, 
        doc="returns list of compound Ids" + _docIds)

    def _get_drug(self):
        if self._drug == None:
            self._drug =  self._get_database("drug", 0)
        return self._drug
    drugIds = property(_get_drug, doc="returns list of drug Ids" + _docIds)

    # set the default organism used by pathways retrieval
    def _get_organism(self):
        return self._organism
    def _set_organism(self, organism):
        if organism in self.organismIds:
            self._organism = organism
            self._pathway = None
            self._module = None
            self._ko = None
            self._glycan = None
            self._compound = None
            self._enzyme = None
            self._drug = None
            self._reaction = None
            self._brite = None
        else:
            self.logging.error("Invalid organism. Check the list in :attr:`organismIds` attribute")
            raise 
    organism = property(_get_organism, _set_organism, doc="returns the current default organism ")

    def _get_pathways(self):
        if self._organism == None:
            self.logging.warning("You must set the organism first (e.g., self.organism = 'hsa')")
            return

        if self._pathway == None:
            res = self.request(self.url + "/list/pathway/%s" % self.organism)
            orgs = [x.split()[0] for x in res.split("\n") if len(x)]
            self._pathway = orgs[:]
        return self._pathway
    pathwayIds = property(_get_pathways, doc="""returns list of pathway Ids for the default organism. 
        
    :attr:`organism` must be set.
    ::

        s = Kegg()
        s.organism = "hsa"
        s.pathwayIds

    """)

    def _get_modules(self):
        if self._organism == None:
            self.logging.warning("You must set the organism first (e.g., self.organism = 'hsa')")
            return

        if self._module == None:
            res = self.request(self.url + "/list/module/%s" % self.organism)
            orgs = [x.split()[0] for x in res.split("\n") if len(x)]
            self._module = orgs[:]
        return self._module
    moduleIds = property(_get_modules, doc="""returns list of module Ids for the default organism. 
        
    :attr:`organism` must be set.
    ::

        s = Kegg()
        s.organism = "hsa"
        s.moduleIds

    """)

    def lookfor_organism(self, query):
        """Look for a specific organism

        :param str query: your search term. upper and lower cases are ignored
        :return: a list of definition that matches the query
        """
        matches = []
        definitions = [" ".join(x.split()) for x in self.list("organism").split("\n")]
        for i, item in enumerate(definitions):
            if query.lower() in item.lower():
                matches.append(i)
        return [definitions[i] for i in matches]

    def lookfor_pathway(self, query):
        """Look for a specific pathway

        :param str query: your search term. upper and lower cases are ignored
        :return: a list of definition that matches the query
        """
        matches = []
        definitions = [" ".join(x.split()) for x in self.list("pathway").split("\n")]
        for i, item in enumerate(definitions):
            if query.lower() in item.lower():
                matches.append(i)
        return [definitions[i] for i in matches]

    def get_pathway_by_gene(self, gene, organism):
        """Search for pathways that contain a specific gene

        :param str gene: a valid gene Id
        :param str organism: a valid organism (e.g., hsa)
        :return: list of pathway Ids that contain the gene

        ::

            >>> s.get_pathway_by_gene("7535", "hsa")
            ['path:hsa04064', 'path:hsa04650', 'path:hsa04660', 'path:hsa05340']

        """ 
        p = KeggParser(verbose=False)
        p.organism = organism
        res = self.get(":".join([organism, gene]))
        dic = p.parse(res)
        if 'pathway' in dic.keys():
            return dic['pathway']
        else:
            print("No pathway found ?")


    def _old_get_pathway_by_gene(self, gene, organism):
        matches = []
        p = KeggParser()
        # using existing buffered list if same organism are used
        if self.organism == organism:
            Ids = self.pathwayIds
        else:
            p.organism = organism
            Ids = p.pathwayIds

        for i,Id in enumerate(Ids): 
            print("scanning %s (%s/%s)" % (Id, i, len(Ids)))
            if Id not in self._buffer.keys():
                res = p.get(Id)
                self._buffer[Id] = res
            else:
                res = self._buffer[Id]

            parsed = p.parse(res)
            if 'gene' in parsed.keys() and gene in parsed['gene']:
                print("Found %s in pathway Ids %s" % (gene, Id))
                matches.append(Id)
        return matches
 
    def parse_kgml_pathway(self, pathwayId, res=None):
        """Parse the pathway in KGML format and returns a dictionary (relations and entries)

        :param str pathwayId: a valid pathwayId e.g. hsa04660
        :param str res: if you already have the output of the query
            get(pathwayId), you can provide it, otherwise it is queried.
        :return: a tuple with the first item being a list of relations. Each
            relations is a dictionary with id2, id2, link, value, name. The
            second item is a dictionary that maps the Ids to 


        ::

            >>> res = s.parse_kgml_pathway("hsa04660")
            >>> set([x['name'] for x in res['relations']])
            >>> res['relations'][-1]
            {'entry1': u'15',
             'entry2': u'13',
             'link': u'PPrel',
             'name': u'phosphorylation',
             'value': u'+p'}

            >>> set([x['link'] for x in res['relations']])
            set([u'PPrel', u'PCrel'])

            >>> res['entries'][4]

        .. seealso:: `KEGG API <http://www.kegg.jp/kegg/xml/docs/>`_
        """
        output = {'relations':[], 'entries':[]}
        if res==None:
            res = self.easyXML(self.get(pathwayId, "kgml"))
        # here entry1 and 2 are Id related to the kgml file

        # read and parse the entries
        entries = [x for x in res.findAll("entry")]
        for entry in entries:

            output['entries'].append({
                'id': entry.get("id"),
                'name': entry.get("name"),
                'type': entry.get("type"),
                'link': entry.get("link"),
                'gene_names': entry.find("graphics").get("name")
                })

        relations = [(x.get("entry1"), x.get("entry2"), x.get("type")) for x in res.findAll("relation")]
        subtypes = [x.findAll("subtype") for x in res.findAll("relation")]

        assert len(subtypes) == len(relations)

        for relation, subtype in zip(relations, subtypes):
            if len(subtype)==0: # nothing to do with the species ??? TODO
                pass
            else:
                for this in subtype:
                    value = this.get("value")
                    name = this.get("name")
                    output['relations'].append({
                        'entry1':relation[0],
                        'entry2':relation[1],
                        'link':relation[2],
                       'value':value, 
                        'name':name})
        # we need to map back to KEgg IDs...
        return output

    def pathway2sif(self, pathwayId, uniprot=True):
        """Extract protein-protein interaction from KEGG pathway to a SIF format

        .. warning:: experimental Not tested on all pathway. should be move to
            another package such as cellnopt

        :param str pathwayId: a valid pathway Id
        :param bool uniprot: convert to uniprot Id or not (default is True)
        :return: a list of relations (A 1 B) for activation and (A -1 B) for
            inhibitions

        This is longish due to the conversion from KeggIds to UniProt.

        This method can be useful to provide prior knowledge network to software
        such as CellNOpt (see http://www.cellnopt.org)
        """
        res = self.parse_kgml_pathway(pathwayId)
        sif = []
        for rel in res['relations']:
            # types can be PPrel (protein-protein interaction only
            if rel['link'] != 'PPrel':
                continue
            if rel['name'] == 'activation':
                Id1 = rel['entry1']
                Id2 = rel['entry2']
                name1 = res['entries'][[x['id'] for x in res['entries']].index(Id1)]['name']
                name2 = res['entries'][[x['id'] for x in res['entries']].index(Id2)]['name']
                type1 = res['entries'][[x['id'] for x in res['entries']].index(Id1)]['type']
                type2 = res['entries'][[x['id'] for x in res['entries']].index(Id2)]['type']
                print("names:", rel, name1, name2)
                print(type1, type2)
                if type1!='gene' or type2!='gene':
                    continue
                if uniprot:
                    name1 = self.conv("uniprot", name1)[1][0]
                    name2 = self.conv("uniprot", name2)[1][0]
                print(name1, 1, name2)
                sif.append([name1, 1, name2])
            elif  rel['name'] == 'inhibition':
                Id1 = rel['entry1']
                Id2 = rel['entry2']
                name1 = res['entries'][[x['id'] for x in res['entries']].index(Id1)]['name']
                name2 = res['entries'][[x['id'] for x in res['entries']].index(Id2)]['name']
                type1 = res['entries'][[x['id'] for x in res['entries']].index(Id1)]['type']
                type2 = res['entries'][[x['id'] for x in res['entries']].index(Id2)]['type']
                print("names:", rel, name1, name2)
                print(type1, type2)
                if type1!='gene' or type2!='gene':
                    continue
                if uniprot:
                    name1 = self.conv("uniprot", name1)[1][0]
                    name2 = self.conv("uniprot", name2)[1][0]
                print(name1, -1, name2)
                sif.append([name1, -1, name2])
            else:
                print("#", rel['entry1'], rel['name'], rel['entry2'])

        return sif

    def __str__(self):
        txt = self.info()
        return txt


class KeggParser(Kegg):
    """This is an extension of the :class:`Kegg` class to ease parsing of dbentries

    This class provides a generic method :meth:`parse` that will read the output 
    of a dbentry returned by :meth:`Kegg.get` and converts it into a dictionary ready to use.

    The :meth:`parse` method is a dispatcher so you do not have to worry about the 
    type of entry you are using. It can be a pathway, a gene, a compound...
    ::

        from bioservices import *
        s = KeggParser()

        # Retrieve a KEGG entry
        res = s.get("hsa04150")

        # parse it
        d = s.parse(res)

    As a pedagogical example, you can then further process this dictionary. Here below, we convert
    the gene Ids found in the pathway into UniProt Ids::

        # Get the KEGG Ids in the pathway
        kegg_geneIds = [x for x in d['gene']]

        # Convert them
        db_up, db_kegg = s.conv("hsa", "uniprot")

        # Get the corresponding uniprot Ids
        indices = [db_kegg.index("hsa:%s" % x ) for x in kegg_geneIds]
        uniprot_geneIds = [db_up[x] for x in indices]

    However, you could also have done it simply as follows::

        kegg_geneIds = [x for x in d['gene']]
        uprot_geneIds = [s.parse(s.get("hsa:"+str(e)))['dblinks']["UniProt:"] for e in d['gene']]

    .. note:: The 2 outputs are slightly different. 

    .. seealso:: http://www.kegg.jp/kegg/rest/dbentry.html
    """
    def __init__(self, verbose=False):
        super(KeggParser, self).__init__(verbose=verbose)

    def parse(self, res):
        """A dispatcher to parse all outputs returned by :meth:`Kegg.get`

        :param str res: output of a :meth:`Kegg.get`.
        :return: a dictionary

        ::
        
            res = s.get("md:hsa_M00554")
            d = s.parse(res)
        """
        entry = res.split("\n")[0].split()[0]
        if entry == "ENTRY":
            dbentry = res.split("\n")[0].split(None, 2)[2]

        if "Pathway" in dbentry and "Module" not in dbentry:
            parser = self.parsePathway(res)
        elif dbentry.lower() == "pathway   module":
            parser = self.parseModule(res)
        elif "Drug" in dbentry: # can be Drug or "Mixture Drug"
            parser = self.parseDrug(res)
        elif "Disease" in dbentry: 
            parser = self.parseDisease(res)
        elif "Environ" in dbentry: 
            parser = self.parseEnviron(res)
        elif "Orthology" in dbentry: 
            parser = self.parseOrthology(res)
        elif "KO" in dbentry: 
            parser = self.parseOrthology(res)
        elif "Genome" in dbentry: 
            parser = self.parseGenome(res)
        elif "gene" in dbentry: 
            parser = self.parseGene(res)
        elif "CDS" in dbentry: 
            parser = self.parseGene(res)
        elif "Compound" in dbentry: 
            parser = self.parseCompound(res)
        elif "Glycan" in dbentry: 
            parser = self.parseGlycan(res)
        elif "Reaction" in dbentry: 
            parser = self.parseReaction(res)
        elif "RPair" in dbentry: 
            parser = self.parseRpair(res)
        elif "RClass" in dbentry: 
            parser = self.parseRclass(res)
        elif "Enzyme" in dbentry: 
            parser = self.parseEnzyme(res)
        else:
            raise NotImplementedError("Entry %s not yet implemented" % dbentry)
        return parser

    def parseDrug(self, res):
        """Parses a drug entry

        ::

            >>> res = s.get("dr:D00001")
            >>> d = s.parseDrug(res)
        """
        flatfile = ["ENTRY", "NAME", "PRODUCTS", "FORMULA", "EXACT_MASS",
            "MOL_WEIGHT", "COMPONENT", "SEQUENCE", "SOURCE", "ACTIVITY",
            "REMARK", "COMMENT", "TARGET", "METABOLISM", "INTERACTION",
           "PATHWAY","STR_MAP","BRITE","DBLINKS", "ATOM", "BOND", "BRACKET"]
        parser = self._parse(res, flatfile)
        return parser

    def parsePathway(self, res):
        """Parses a pathway entry

        ::

            >>> res = s.get("path:hsa10584")
            >>> d = s.parsePathway(res)

        """
        flatfile = ["ENTRY", "NAME", "DESCRIPTION", "CLASS", "PATHWAY_MAP", 
            "MODULE", "DISEASE", "DRUG", "DBLINKS", "ORGANISM", "ORTHOLOGY", 
            "GENE", "ENZYME", "REACTION", "COMPOUND", "REFERENCE", 
            "REL_PATHWAY", "KO_PATHWAY"]
        parser = self._parse(res, flatfile)
        return parser

    def parseModule(self, res):
        """Parses a module entry

        ::

            >>> res = s.get("md:hsa_M00554")
            >>> d = s.parseModule(res)
        """
        flatfile = ["ENTRY", "NAME", "DEFINITION",  "PATHWAY",
            "ORTHOLOGY", "CLASS", "BRITE", "ORGANISM", "GENE", "REACTION",
            "COMPOUND", "COMMENT", "DBLINKS", "REFERENCE", "REF_MODULE"]
        parser = self._parse(res, flatfile)
        return parser

    def parseDisease(self, res):
        """Parses a disease entry

        ::

            >>> res = s.get("ds:H00001")
            >>> d = s.parseDisease(res)
        """
        flatfile = ["ENTRY", "NAME", "DESCRIPTION", "CATEGORY", "PATHWAY", "GENE",
            "ENV_FACTOR", "MARKER", "DRUG", "COMMENT", "DBLINKS", "REFERENCE"]
        parser = self._parse(res, flatfile)
        return parser

    def parseEnviron(self, res):
        """Parses a environ entry
        ::

            >>> res = s.get("ev:E00001")
            >>> d = s.parseEnviron(res)
        """
        flatfile = ['ENTRY', "NAME", "CATEGORY", "COMPONENT", "SOURCE", 
            "REMARK", "COMMENT", "BRITE", "DBLINKS"]
        parser = self._parse(res, flatfile)
        return parser

    def parseOrthology(self, res):
        """Parses Orthology entry

        ::
        
            >>> res = s.get("ko:K00001")
            >>> d = s.parseOrthology(res)

        .. note:: in other case genes key is "gene". Here it is "genes".
        """
        flatfile = ['ENTRY', "NAME", "DEFINITION", "PATHWAY", "MODULE", 
            "DISEASE", "BRITE", "DBLINKS", "GENES", "REFERENCE"]
        parser = self._parse(res, flatfile)
        return parser

    def parseGenome(self, res):
        """Parses a Genome entry

        ::

            >>> res = s.get('genome:T00001')
            >>> d = s.parseGenome(res)

        """

        flatfile = ["ENTRY", "NAME", "DEFINITION", "ANNOTATION", "TAXONOMY", 
            "DATA_SOURCE", "ORIGINAL_DB", "KEYWORDS", "DISEASE", "COMMENT", 
            "CHROMOSOME", "PLASMID", "STATISTICS", "REFERENCE"]
        parser = self._parse(res, flatfile)
        return parser

    def parseGene(self, res):
        """Parses a gene entry

        ::

            >>> res = s.get("hsa:1525")
            >>> d = s.parseGene(res)

        """

        flatfile = ["ENTRY", "NAME", "DEFINITION", "ORTHOLOGY", "ORGANISM",
            "PATHWAY", "MODULE", "DISEASE", "DRUG_TARGET", "CLASS", "MOTIF", "DBLINKS", 
            "STRUCTURE", "POSITION", "AASEQ", "NTSEQ"]
        parser = self._parse(res, flatfile)
        return parser


    def parseCompound(self, res):
        """Parses a compound entry

        ::

            >>> s.get("cpd:C00001")
            >>> d = s.parseCompound(res)
        """
        flatfile = ["ENTRY", "NAME", "FORMULA", "EXACT_MASS", "MOL_WEIGHT",
            "SEQUENCE", "REMARK", "COMMENT", "REACTION", "PATHWAY", "ENZYME", "BRITE",
            "REFERENCE", "DBLINKS", "ATOM", "BOND", "BRACKET"]
        parser = self._parse(res, flatfile)
        return parser

    def parseGlycan(self, res):
        """Parses a glycan entry

        ::

            >>> res = s.get("gl:G00001")
            >>> d = s.parseGlycan(res)
        """
        flatfile = ["ENTRY", "NAME", "COMPOSITION", "MASS", "CLASS", "REMARK",
            "COMMENT", "REACTION", "PATHWAY", "ENZYME", "ORTHOLOGY", 
            "REFERENCE", "DBLINKS", "NODE", "EDGE", "BRACKET"]
        parser = self._parse(res, flatfile)
        return parser

    def parseReaction(self, res):
        """Parses a reaction entry

        ::

            >>> res = s.get("rn:R00001")
            >>> d = s.parseReaction(res)
        """
        flatfile = ["ENTRY", "NAME", "DEFINITION", "EQUATION", "REMARK",
            "COMMENT", "RPAIR", "ENZYME", "PATHWAY", "ORTHOLOGY", "REFERENCE"]
        parser = self._parse(res, flatfile)
        return parser

    def parseRpair(self, res):
        """Parses a rpair entry

        ::

            >>> res = s.get("rp:RP00001")
            >>> d = s.parseRpair(res)


        .. todo:: a better parsing
        """
        flatfile = ["ENTRY", "NAME", "COMPOUND", "TYPE", "RDM", "RCLASS",
            "RELATEDPAIR", "REACTION", "Enzyme  ENZYME", "ALIGN", "ENTRY1", "ENTRY2"]
        parser = self._parse(res, flatfile)
        return parser


    def parseRclass(self, res):
        """Parses a rclass entry

        ::

            >>> res = s.get("rc:RC00001")
            >>> d = s.parseRclass(res)

        .. todo:: a better parsing
        """
        flatfile = ["ENTRY", "DEFINITION", "RPAIR", "REACTION",
            "ENZYME","PATHWAY", "ORTHOLOGY"]
        parser = self._parse(res, flatfile)
        return parser

    def parseEnzyme(self, res):
        """Parses an enzyme entry

        ::

            >>> res = s.get('ec:1.1.1.1')
            >>> d = s.parseEnzyme(res)
        """
        flatfile = ["ENTRY", "NAME", "CLASS", "SYSNAME", "REACTION", "ALL_REAC",
            "SUBSTRATE", "PRODUCT", "COMMENT", "PATHWAY", "ORTHOLOGY", "GENES", 
            "REFERENCE"]
        parser = self._parse(res, flatfile)
        return parser

    def _parse(self, res, flatfile):
        """Reaction is currently a dictionary if more than one line maybe we
        want a list instead.


        """
        output = {}
        lines = res.split("\n")

        # scanning res and searching for flatfile keywords; keep track of
        # current one. Should be an entry first.
        current = None
        countref = 1 # reference counter for reference without a value
        for line in lines:
            line = line.strip()
            if line == "///" or len(line)==0:
                continue

            if len(line.split())>=2:# split each line to search for the first term
                key, text = line.split(None,1)
            else:
                key = line

            # REFERENCES are dealt with in the else
            if key in flatfile and \
                key not in ["REFERENCE", "AUTHORS", "TITLE", "JOURNAL"]:
                # store the first appearance of a key
                output[key.lower()] = line.split(key)[1].strip()
                current = key.lower()   # keep track of the current key
            else:
                # There are many instances of referneces that are followed
                # by authors/title/journal triplet
                if key == "REFERENCE":
                    current = key.lower()
                    if current not in output.keys():
                        # the first time, we create a dictionary
                        output[current] = {}

                # Sometimes, we want to create a dictionary. For instance, genes
                # but in other cases  we just want to append the text (e.g.
                # remarks)
                if current in ["gene", "reference", "rel_pathway", "orthology",\
                        "pathway","reaction", "compound", "dblinks", "marker"]:
                    if isinstance(output[current], dict) == False:
                        # The item may be of different length. In the gene case, we
                        # want to provide a list of dictionaries with key being the
                        # gene id but in some other cases, 
                        try:
                            key, value = output[current].split(" ",1)
                        except:
                            value = output[current]
                        output[current] = {key:value}
                    mode = "dict"
                else:
                    if isinstance(output[current], list) == False:
                        value = output[current]
                        output[current] = [value]
                    mode = "append"

                # For references only
                if current == "reference":
                    try:
                        key, value = line.strip().split(" ",1)
                    except:
                        key = line.strip()
                        countref += 1
                        value = str(countref)
                    if key.upper() in ["AUTHORS", "TITLE", "JOURNAL"]:
                        output[current][pubmed][key] = value
                    elif key.upper() == "REFERENCE":
                        pubmed = value.strip()
                        if pubmed in output[current].keys():
                            pubmed += "_" + str(countref)
                            countref += 1
                        output[current][pubmed] = {}
                else: # and all others
                    if mode == "dict":
                        try:
                            key, value = line.strip().split(" ",1)
                        except: # needed in INTERACTION case (DRUG)
                            key = line.strip()
                            value = ""
                        output[current][key.strip()] = value.strip()
                    else:
                        output[current].append(line)

        # some cleanup
        if "ntseq" in output.keys():
            data = output['ntseq']
            output['ntseq'] = {data[0]: reduce(lambda x,y:x+y, data[1:])}
        if "aaseq" in output.keys():
            data = output['aaseq']
            output['aaseq'] = {data[0]: reduce(lambda x,y:x+y, data[1:])}
        return output




